#include "azeq6.h"
#include "GL/glu.h"
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846

#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)
#define DEUXPI 6.28318530717958647692
#define PIS2 1.57079632679489661923
#include "mllglutil.h"

#if 0
static void madvec(double* w, double* a, double* v)
{
    double p, q ;
    p  = a[0] * v[0] + a[3] * v[1] + a[6] * v[2];
    q  = a[1] * v[0] + a[4] * v[1] + a[7] * v[2];
    w[2] = a[2] * v[0] + a[5] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
}

static void madmad(double* a,double* b, double* c)
{
    register int i ;
    double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[1] + b[6] * c[2];
    d[1] = b[1] * c[0] + b[4] * c[1] + b[7] * c[2];
    d[2] = b[2] * c[0] + b[5] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[3] * c[4] + b[6] * c[5];
    d[4] = b[1] * c[3] + b[4] * c[4] + b[7] * c[5];
    d[5] = b[2] * c[3] + b[5] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[3] * c[7] + b[6] * c[8];
    d[7] = b[1] * c[6] + b[4] * c[7] + b[7] * c[8];
    a[8] = b[2] * c[6] + b[5] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
}

static void mrotx(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[0] = 1.;
    a[1] = a[2] = a[3] = a[6] = 0.;
    a[4] = a[8]= c;
    a[5] = s; a[7] = -s;
}

static void mroty(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[4] = 1.;
    a[1] = a[3] = a[5] = a[7] = 0.;
    a[0] = a[8]= c;
    a[6] = s; a[2] = -s;
}

static void mrotz(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[8] = 1.;
    a[2] = a[5] = a[6] = a[7] = 0.;
    a[0] = a[4]= c;
    a[1] = s; a[3] = -s;
}

static double *mat44from33(double *m44, double *m33)
{
    m44[0] = m33[0] ; m44[4] = m33[3] ;  m44[8]  = m33[6] ; m44[12] = 0. ;
    m44[1] = m33[1] ; m44[5] = m33[4] ;  m44[9]  = m33[7] ; m44[13] = 0. ;
    m44[2] = m33[2] ; m44[6] = m33[5] ;  m44[10] = m33[8] ; m44[14] = 0. ;
    m44[3] = 0.     ; m44[7] = 0.     ;  m44[11] = 0.     ; m44[15] = 1. ;
    return m44 ;
}
#endif

// Calcul coordonnées avec les contre-poids en bas, cad avec Ah dans [12h - 18h - 24h]
// soit en degres Ah > 180.
// Renvoie 1 si on a flippé, et 0 si conservé l'angle horaire.
// Angles en degrés
int cwdown_dg(double *aziStd, double *sitStd, double aziIn, double sitIn)
{
    int flip = 0;
    // Ramène l'azimut dans [0 , 360] pour le test
    while(aziIn > 360.) aziIn -= 360.;
    while(aziIn <= 0.) aziIn += 360.; // met 0 à 360 pour ne pas flipper sur 0
    // Si Ah < 12h, cad à 180, on flippe
    if (aziIn < 180.)
    {
        aziIn += 180.;
        sitIn = 180. - sitIn;
        flip = 1;
    }
    // Ramène le site dans [-180 , +180]
    while(sitIn > 180.) sitIn -= 360.;
    while(sitIn < -180.) sitIn += 360.;

    *aziStd = aziIn;
    *sitStd = sitIn;
    return flip;
}

// Calcul coordonnées avec les contre-poids en bas, c'ad avec Ah dans [12h - 18h - 24h]
// soit en degres Ah > 180.
// Renvoie 1 si on a flippé, et 0 si conservé l'angle horaire.
// Angles en radians
int cwdown_rd(double *aziStd, double *sitStd, double aziIn, double sitIn)
{
    int flip = 0;
    // Ramène l'azimut dans [0 , 360] pour le test
    while(aziIn > DEUXPI) aziIn -= DEUXPI;
    while(aziIn <= 0.) aziIn += DEUXPI; // met 0 à 360 pour ne pas flipper sur 0
    // Si Ah < 12h, cad à 180, on flippe
    if (aziIn < M_PI)
    {
        aziIn += M_PI;
        sitIn = M_PI - sitIn;
        flip = 1;
    }
    // Ramène le site dans [-180 , +180]
    while(sitIn > M_PI) sitIn -= DEUXPI;
    while(sitIn < -M_PI) sitIn += DEUXPI;

    *aziStd = aziIn;
    *sitStd = sitIn;
    return flip;
}

// Test si en-dessous de l'horizon :
// Renvoie : 0 si au-dessus, 1 si en-dessous
// angles en degrés
int underH_dg(double ah, double dec, double lat)
{
    ah = ah*DEG2RAD; dec = dec*DEG2RAD; lat = lat*DEG2RAD;
    double z = cos(lat)*cos(ah)*cos(dec) + sin(lat)*sin(dec);
    return z >= 0. ? 0 : 1;
}

// Test si en-dessous de l'horizon : 0 si au-dessus, 1 si en-dessous
// Renvoie : 0 si au-dessus, 1 si en-dessous
// Angles en radians
int underH_rd(double ah, double dec, double lat)
{
    double z = cos(lat)*cos(ah)*cos(dec) + sin(lat)*sin(dec);
    return z >= 0. ? 0 : 1;
}

// Conversion coordonnées polaires non standard en coordonnées polaires standard
// c'est-à-dire avec la valeur absolue du site < à 90
// version en degres
int cp2std_dg(double *aziStd, double *sitStd, double aziIn, double sitIn)
{
    int flip = 0;
    // Ramene le site dans [-180 , +180]
    while(sitIn > 180.) sitIn -= 360.;
    while(sitIn < -180.) sitIn += 360.;
    // Ramène le site inférieur à -90
    if (sitIn > 90)
    {
        aziIn += 180.; sitIn = 180. - sitIn; flip = 1;
    }
    // Ramène le site supérieur à -90
    if (sitIn < -90)
    {
        aziIn += 180.; sitIn = -180. - sitIn; flip = 1;
    }
    // Ramène l'azimut dans [0 , 360]
    while(aziIn > 360.) aziIn -= 360.;
    while(aziIn < 0.) aziIn += 360.;
    *aziStd = aziIn;
    *sitStd = sitIn;
    return flip;
}

// Conversion coordonnées polaires non standard en coordonnées polaires standard
// c'est-à-dire avec la valeur absolue du site < à 90
// version en radians
int cp2std_rd(double *aziStd, double *sitStd, double aziIn, double sitIn)
{
    int flip = 0;
    // Ramene le site dans [-PI , +PI]
    while(sitIn > M_PI) sitIn -= DEUXPI;
    while(sitIn < -M_PI) sitIn += DEUXPI;
    // Ramène le site inférieur à -90
    if (sitIn > PIS2)
    {
        aziIn += M_PI; sitIn = M_PI - sitIn; flip = 1;
    }
    // Ramène le site supérieur à -90
    if (sitIn < -PIS2)
    {
        aziIn += M_PI; sitIn = -M_PI - sitIn; flip = 1;
    }
    // Ramène l'azimut dans [-180 , +180]
    while(aziIn > M_PI) aziIn -= DEUXPI;
    while(aziIn < -M_PI) aziIn += DEUXPI;
    *aziStd = aziIn;
    *sitStd = sitIn;
    return flip;
}

// Conversion degres decimaux en HH MM SS.S et string
// Usage : str = deg2hms(buf, ang);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en degre
// Sortie : str = HHhMMmSS.Ss
char *deg2strhms(char *buffer, double ang)
{
int h, m;
double s, hh, mm;
long jours;
    hh = ang/15. + 5.e-9; // +1.8e-5 secondes de temps pour obliger à passer les caps.
    // On enlève les jours et on ramène entre 0 et 24h.
    if (hh < 0.) { hh = -hh; jours = (long) (hh/24.); hh = 24. - (hh - 24.*jours);}
    else { jours = (long) (hh/24.); hh = hh - 24.*jours;}
    h = (int) hh;
    mm = (hh - h)*60.0; m = (int) mm;
    s = (mm - m)*60.;
    sprintf(buffer,"%02dh%02dm%03.0lfs",h,m,s);
    return buffer;
}

// Conversion degres decimaux en HH MM SS.S et string
// Usage : str = deg2hms(buf, ang, &hh, &mhm, &ss);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en degre
// Sortie : str = HHhMMmSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *deg2hms(char *buffer, double ang, int *h, int *m, double *s)
{
double hh, mm;
long jours;
    hh = ang/15. + 5.e-9; // +1.8e-5 secondes de temps pour obliger à passer les caps.
    // On enlève les jours et on ramène entre 0 et 24h.
    if (hh < 0.) { hh = -hh; jours = (long) (hh/24.); hh = 24. - (hh - 24.*jours);}
    else { jours = (long) (hh/24.); hh = hh - 24.*jours;}
    *h = (int) hh;
    mm = (hh - *h)*60.0; *m = (int) mm;
    *s = (mm - *m)*60.;
    sprintf(buffer,"%02dh%02dm%03.0lfs",*h,*m,*s);
    return buffer;
}

Azeq6::Azeq6()
{
    dimConf = 4;
    //#define ELOING 300.
    //#define SITE 20.0
    //#define AZIMUT 60.0
    //    DistCam = DistCamIni = ELOING; /* de l'observateur */
    //    AziCam = AziCamIni = AZIMUT;
    //    SitCam = SitCamIni = SITE; /* Direction de l'observateur */
    delete labelHelp;
    labelHelp = new QLabel(
                tr(
                    "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p     : projection orthogonale</li>"
                    "<li>P     : perspective</li>"
                    "<li>i-I   : Affichage Infos</li></ul>"
                    "<h5>Déplacements scène :</h5>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>r-R             : rotation azimut éclairage</li>"
                    "<li>s-S             : rotation site éclairage</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h5>Mouvements monture :</h5>"
                    "<ul><li>0    : Angles mis à 0</li>"
                    "<li>o-O  : Configuration origine (Home)</li>"
                    "<li>h-H  : Rotation en angle Horaire</li>"
                    "<li>d-D  : Rotation en Déclinaison</li>"
                    "<li>l-L  : Réglage Latitude (site)</li>"
                    "<li>a-A  : Rotation en Azimut(/nord)</li></ul>"
                    "<h5>En cours de programmation (à terminer) :</h5>"
                    "<ul><li>w-W    : Appel boite de dialogue du GoTo</li>"
                    "<li>g-G    : Lancement déplacement GoTo</li>"
                    "<li>Espace : Suspension/reprise déplacement GoTo</li></ul>"
                    ), 0);
    labelHelp->setWindowTitle("Aide Azeq6");

    DeltaA = 1.;
    timerGo = new QTimer(this);
    timerGo->setSingleShot(false);
    connect(timerGo, SIGNAL(timeout()), this, SLOT(gotoEnd()));
    deltaT = 20; // intervalle rafraichissement en ms
    timerGo->setInterval(deltaT);
    fgoto = false;
    flagEnd = false; // Le but est sigDep
    fpause = false;
    flagInfos = true;

    goDialog = new GoDialog(this, this);
//    goDialog->setModal(false);
}

static GLfloat faible[] = {0.2f, 0.2f, 0.2f, 1.0f};
static GLfloat puissante[] = {1.0f, 1.0f, 1.0f, 1.0f};

// Coefficients de reflection de quelques materiaux
static GLfloat plastocNoir_amb[] = {0.0f, 0.0f, 0.0f, 1.f};
static GLfloat plastocNoir_dif[] = {0.01f, 0.01f, 0.01f, 1.f};
static GLfloat plastocNoir_spc[] = {0.5f, 0.5f, 0.5f, 1.f};
static GLfloat plastocNoir_shi = 32.f;
static GLfloat cuivre_amb[] = {0.32941f, 0.22353f,  0.02745f, 1.f};
static GLfloat cuivre_dif[] = {0.78039f, 0.56863f, 0.11373f, 1.f};
static GLfloat cuivre_spc[] = {0.99216f, 0.94118f, 0.80784f, 1.f};
static GLfloat cuivre_shi = 27.897f;
static GLfloat bronze_amb[] = {0.2125f, 0.1275f, 0.054f, 1.f};
static GLfloat bronze_dif[] = {0.714f, 0.4284f, 0.18144f, 1.f};
static GLfloat bronze_spc[] = {0.39355f, 0.27191f, 0.16672f, 1.f};
static GLfloat bronze_shi = 25.6f;
static GLfloat chrome_amb[] = {0.25f, 0.25f, 0.25f, 1.f};
static GLfloat chrome_dif[] = {0.4f, 0.4f, 0.4f, 1.f};
static GLfloat chrome_spc[] = {0.7746f, 0.7746f, 0.7746f, 1.f};
static GLfloat chrome_shi = 76.8f;
static GLfloat cuivreRed_amb[] = {0.19125f, 0.0735f, 0.0225f, 1.f};
static GLfloat cuivreRed_dif[] = {0.7038f, 0.27048f,  0.0828f, 1.f};
static GLfloat cuivreRed_spc[] = {0.25678f, 0.13762f, 0.08601f, 1.f};
static GLfloat cuivreRed_shi = 12.8f;
static GLfloat gold_amb[] = {0.24725f, 0.1995f, 0.0745f, 1.f};
static GLfloat gold_dif[] = {0.75164f, 0.60648f, 0.22648f, 1.f};
static GLfloat gold_spc[] = {0.62828f, 0.5558f, 0.36607f, 1.f};
static GLfloat gold_shi = 51.2f;
static GLfloat etain_amb[] = {0.10588f, 0.05882f, 0.11373f, 1.f};
static GLfloat etain_dif[] = {0.42745f, 0.47059f, 0.54118f, 1.f};
static GLfloat etain_spc[] = {0.3333f, 0.3333f, 0.52157f, 1.f};
static GLfloat etain_shi = 9.84615f;
static GLfloat argent_amb[] = {0.19225f, 0.19225f, 0.19225f, 1.f};
static GLfloat argent_dif[] = {0.50754f, 0.50754f, 0.50754f, 1.f};
static GLfloat argent_spc[] = {0.50827f, 0.50827f, 0.50827f, 1.f};
static GLfloat argent_shi = 51.2f;
static GLfloat argentBri_amb[] = {0.23125f, 0.23125f, 0.23125f, 1.f};
static GLfloat argentBri_dif[] = {0.2775f, 0.2775f, 0.2775f, 1.f};
static GLfloat argentBri_spc[] = {0.77391f, 0.77391f, 0.77391f, 1.f};
static GLfloat argentBri_shi = 89.6f;
static GLfloat miroir_amb[] = {0.19225f, 0.19225f, 0.19225f, 1.f};
static GLfloat miroir_dif[] = {0.2775f, 0.2775f, 0.2775f, 1.f};
static GLfloat miroir_spc[] = {0.95f, 0.95f, 0.95f, 1.f};
static GLfloat miroir_shi = 95.f;
static GLfloat clair_amb[] = {0.95f, 0.95f, 0.95f, 1.f};
static GLfloat clair_dif[] = {0.2775f, 0.2775f, 0.2775f, 1.f};
static GLfloat clair_spc[] = {0.50827f, 0.50827f, 0.50827f, 1.f};
static GLfloat clair_shi = 25.f;


// rendus de divers materiaux
static GLfloat *ambient[] = {plastocNoir_amb, cuivre_amb, bronze_amb, chrome_amb,
                             cuivreRed_amb, gold_amb, etain_amb, argent_amb, argentBri_amb, miroir_amb, clair_amb};
static GLfloat *diffuse[] = {plastocNoir_dif, cuivre_dif, bronze_dif, chrome_dif,
                             cuivreRed_dif, gold_dif, etain_dif, argent_dif, argentBri_dif, miroir_dif, clair_dif};
static GLfloat *specular[] = {plastocNoir_spc, cuivre_spc, bronze_spc, chrome_spc,
                              cuivreRed_spc, gold_spc, etain_spc, argent_spc, argentBri_spc, miroir_spc, clair_spc};
static GLfloat shininess[] = {plastocNoir_shi, cuivre_shi, bronze_shi, chrome_shi,
                              cuivreRed_shi, gold_shi, etain_shi, argent_shi, argentBri_shi, miroir_shi, clair_shi};

void selectMaterial (int k)
{
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient[k]);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse[k]);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular[k]);
    glMaterialf(GL_FRONT, GL_SHININESS, shininess[k]);
}

void Azeq6::create()
{
#define NDIV 40
    const unsigned char colGrisclair[]={220,220,220};
    const unsigned char colGrisfonce[]={100,100,100};
    const unsigned char colPlatform[]={250,200,50};
    const unsigned char colGrenat[]={200,000,000};
    size = 0.03;

    GLUquadricObj *quadric;

    double pt[16][3] = {
        { 10.*size, 4.*size,  2.*size},
        { 10.*size,-4.*size,  2.*size},
        { 10.*size, 4.*size,-12.*size},
        { 10.*size,-4.*size,-12.*size},
        {-10.*size, 4.*size,-12.*size},
        {-10.*size,-4.*size,-12.*size},
        {-10.*size, 4.*size,  2.*size},
        {-10.*size,-4.*size,  2.*size},
        {-11.*size, 4.*size,  2.*size},
        {-11.*size,-4.*size,  2.*size},
        {-11.*size, 4.*size,-13.*size},
        {-11.*size,-4.*size,-13.*size},
        { 11.*size, 4.*size,-13.*size},
        { 11.*size,-4.*size,-13.*size},
        { 11.*size, 4.*size,  2.*size},
        { 11.*size,-4.*size,  2.*size},
    };



    // Modes sélectionnés par défaut
    glEnable(GL_DEPTH_TEST);	// Zbuffer activé
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL) ; // Les faces sont pleines
    glEnable(GL_CULL_FACE);		// Pré-élimine une des 2 faces
    glCullFace(GL_BACK);        // Ce sera la face arrière
    glFrontFace(GL_CCW);		// La face avant est CCW par défaut
    quadric = gluNewQuadric() ;
    gluQuadricDrawStyle(quadric, GLU_FILL);

    // La colonne
    Colonne = glGenLists(1);
    glNewList(Colonne, GL_COMPILE);
    glTranslated(-20.*size,0.,0.) ;    // Pour éloigner la scène
    selectMaterial(3);
    glShadeModel(GL_SMOOTH); // Normales interpolées
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    glColor3ubv(colGrisfonce) ;
    glPushMatrix();
    glTranslated(0., 0., -71.*size) ;
    selectMaterial(9);
    gluDisk(quadric, 0., 100.*size, 8, 1) ;
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0., 100.*size, 8, 1) ;
    selectMaterial(3);
    //    gluDisk(quadric, 0., 18.*size, NDIV, 1) ;
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 18.*size, 18.*size, 2.*size, NDIV, 1) ; // -66
    glTranslated(0., 0., 2.*size) ;
    gluCylinder(quadric, 18.*size, 10.*size, 8.*size, NDIV, 1) ; // -58
    glTranslated(0., 0., 8.*size) ;
    gluCylinder(quadric, 10.*size, 10.*size, 12.*size, NDIV, 1) ; // -46
    glTranslated(0., 0., 12.*size) ;
    gluCylinder(quadric, 10.*size, 6.*size, 4.*size, NDIV, 1) ; // -42
    glTranslated(0., 0., 4.*size) ;
    gluCylinder(quadric, 6.*size, 6.*size, 30.*size, NDIV, 1) ; // -12
    glTranslated(0., 0., 30.*size) ;
    gluCylinder(quadric, 6.*size, 4.*size, 2.*size, NDIV, 1) ; // -10
    gluCylinder(quadric, 3.*size, 3.*size, 3.5*size, NDIV, 1) ; // -10
    glTranslated(0., 0., 3.5*size) ;
    gluDisk(quadric, 0., 3.*size, NDIV, 1) ;
    glPopMatrix();
    glEndList();

    // Le support du corps de l'EQ6
    Etrier = glGenLists(1);
    glNewList(Etrier, GL_COMPILE);
    selectMaterial(6);
    glShadeModel(GL_FLAT); // Normales pures
    glColor3ubv(colGrisclair) ;
    glBegin(GL_QUADS);
    glNormal3d(-1.,0.,0.);
    glVertex3dv(pt[0]); glVertex3dv(pt[2]); glVertex3dv(pt[3]); glVertex3dv(pt[1]);

    glNormal3d(0.,0.,1.);
    glVertex3dv(pt[2]); glVertex3dv(pt[4]); glVertex3dv(pt[5]); glVertex3dv(pt[3]);

    glNormal3d(1.,0.,0.);
    glVertex3dv(pt[4]); glVertex3dv(pt[6]); glVertex3dv(pt[7]); glVertex3dv(pt[5]);

    glNormal3d(0.,0.,1.);
    glVertex3dv(pt[6]); glVertex3dv(pt[8]); glVertex3dv(pt[9]); glVertex3dv(pt[7]);

    glNormal3d(-1.,0.,0.);
    glVertex3dv(pt[8]); glVertex3dv(pt[10]); glVertex3dv(pt[11]); glVertex3dv(pt[9]);

    glNormal3d(0.,0.,-1.);
    glVertex3dv(pt[10]); glVertex3dv(pt[12]); glVertex3dv(pt[13]); glVertex3dv(pt[11]);

    glNormal3d(1.,0.,0.);
    glVertex3dv(pt[12]); glVertex3dv(pt[14]); glVertex3dv(pt[15]); glVertex3dv(pt[13]);

    glNormal3d(0.,0.,1.);
    glVertex3dv(pt[0]); glVertex3dv(pt[1]); glVertex3dv(pt[15]); glVertex3dv(pt[14]);
    glEnd() ;

    glNormal3d(0.,-1.,0.);
    glBegin(GL_POLYGON);
    glVertex3dv(pt[1]); glVertex3dv(pt[3]); glVertex3dv(pt[13]); glVertex3dv(pt[15]);
    glEnd() ;
    glBegin(GL_POLYGON);
    glVertex3dv(pt[3]); glVertex3dv(pt[5]); glVertex3dv(pt[11]); glVertex3dv(pt[13]);
    glEnd() ;
    glBegin(GL_POLYGON);
    glVertex3dv(pt[5]); glVertex3dv(pt[7]); glVertex3dv(pt[9]); glVertex3dv(pt[11]);
    glEnd() ;

    glNormal3d(0.,1.,0.);
    glBegin(GL_POLYGON);
    glVertex3dv(pt[0]); glVertex3dv(pt[14]); glVertex3dv(pt[12]); glVertex3dv(pt[2]);
    glEnd() ;
    glBegin(GL_POLYGON);
    glVertex3dv(pt[2]); glVertex3dv(pt[12]); glVertex3dv(pt[10]); glVertex3dv(pt[4]);
    glEnd() ;
    glBegin(GL_POLYGON);
    glVertex3dv(pt[4]); glVertex3dv(pt[10]); glVertex3dv(pt[8]); glVertex3dv(pt[6]);
    glEnd() ;

    glEndList();

    // L'axe de hauteur
    SupEq6 = glGenLists(1);/* Un numéro de DisplayList */
    glNewList(SupEq6, GL_COMPILE);/* Définition sans execution */
    selectMaterial(2);
    glShadeModel(GL_FLAT); // Normales pures
    gluQuadricNormals(quadric, GLU_FLAT);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    glPushMatrix();
    glColor3ubv(colGrisfonce) ;
    glRotated(90.,0.,1.,0.) ;
    glTranslated(0.,0.,-13.*size) ;
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0.0, 1.5*size, 6, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 1.5*size, 1.5*size, 26.*size, 6, 1) ;
    glTranslated(0.,0.,26.*size) ;
    gluDisk(quadric, 0.0, 1.5*size, 6, 1);
    glPopMatrix();
    /* Support EQ6 */
    selectMaterial(1);
    glColor3ubv(colGrisclair) ;
    glPushMatrix(); /* Push repere base */
    glTranslated(0., 0., -8.*size);
    glRotated(45.,0.,0.,1.); // Pour avoir un coté perpendiculaire a Ox0
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0.0, 10.*size, 4, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 10.*size, 10.*size, 12.0*size, 4, 1);
    glTranslated(0., 0., 12.*size);
    // La roue A.H coté support
    selectMaterial(7);
    glShadeModel(GL_SMOOTH); // Normales interpolées
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0.0, 10.*size, NDIV, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 10.*size, 10.*size, 1.0*size, NDIV, 1);
    glPopMatrix(); /* Retour repere base */
    glEndList();/* fin de  SupEq6 */

    /* Corps EQ6 */
    CorpsEq6 = glGenLists(1);/* Un numéro de DisplayList */
    glNewList(CorpsEq6, GL_COMPILE);/* Définition sans execution */
    glColor3ubv(colGrisclair) ;
    glPushMatrix(); /* Push repere base */
    glTranslated(0., 0., 5.*size);
    // La roue A.H coté EQ6
    selectMaterial(4);
    glColor3ubv(colGrenat);
    glShadeModel(GL_SMOOTH);
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    gluCylinder(quadric, 10.*size, 10.*size, 1.0*size, NDIV, 1);
    // Le bloc EQ6
    selectMaterial(1);
    glShadeModel(GL_FLAT);
    gluQuadricNormals(quadric, GLU_FLAT);
    gluQuadricOrientation(quadric, GLU_FLAT);
    glTranslated(0., 0., 1.*size);
    glColor3ubv(colPlatform);
    glPushMatrix();
    glRotated(45.,0.,0.,1.); // Pour avoir un coté perpendiculaire a Ox0
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0.0, 15.*size, 4, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 15.*size, 15.*size, 16.*size, 4, 1);
    glTranslated(0., 0., 16.*size);
    gluDisk(quadric, 0.0, 15.*size, 4, 1);
    glPopMatrix();
    // Support tige
    glTranslated(0., -10.61*size, 8.*size);
    glRotated(90.,1.,0.,0.) ;
    selectMaterial(2);
    glShadeModel(GL_SMOOTH);
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    gluCylinder(quadric, 5.*size, 5.*size, 6.*size, NDIV, 1);
    glTranslated(0., 0., 6.*size);
    gluDisk(quadric, 0., 5.*size, NDIV, 1);
    // La tige
    selectMaterial(3);
    glColor3ubv(colGrisclair) ;
    gluCylinder(quadric, 2.*size, 2.*size, 25.*size, NDIV, 1);
    // Un poids
    glTranslated(0., 0., 20.*size);
    selectMaterial(5);
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0., 10.*size, NDIV, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);

    gluCylinder(quadric, 10*size, 10*size, 2.*size, NDIV, 1);
    glTranslated(0., 0., 2.*size);
    gluDisk(quadric, 0., 10.*size, NDIV, 1);
    // Le bout de la tige
    selectMaterial(3);
    glTranslated(0., 0., 3.*size);
    gluDisk(quadric, 0., 2.*size, NDIV, 1);
    // Vers l'autre coté, support tube du télescope
    glTranslated(0., 0., -(2.*10.61+31.)*size);
    // La 1ere partie de la couronne de rotation en déclinaison
    selectMaterial(7);
    glColor3ubv(colGrenat);
    gluDisk(quadric, 0., 10.*size, NDIV, 1);
    glTranslated(0., 0., -1.*size);
    gluCylinder(quadric, 10.*size, 10.*size, 1.0*size, NDIV, 1);
    glPopMatrix(); /* Retour repere base */
    glEndList();/* fin de  CorpsEq6 */

    // La partie télescope
    Scope = glGenLists(1);
    glNewList(Scope, GL_COMPILE);
    glPushMatrix();
    // L'axe de hauteur
    glColor3ubv(colGrisfonce) ;
    glTranslated(0.,  10.61*size, 0.);
    glRotated(-90.,1.,0.,0.) ;
    glTranslated(0.,  0., 1.*size);
    // La 2eme partie de la couronne de rotation en déclinaison
    selectMaterial(4);
    glColor3ubv(colGrisclair) ;
    glShadeModel(GL_SMOOTH);
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    gluCylinder(quadric, 10.*size, 10.*size, 1.0*size, NDIV, 1);
    glTranslated(0., 0., 1.*size);
    gluDisk(quadric, 0., 10.*size, NDIV, 1);
    glShadeModel(GL_FLAT);
    gluQuadricNormals(quadric, GLU_FLAT);
    gluQuadricOrientation(quadric, GLU_FLAT);
    // La queue d'aronde
    selectMaterial(2);
    glColor3ubv(colGrenat);
    glTranslated(0., 0.*size, 1.*size);
    glRotated(90.,1.,0.,0.); // remet z vertical
    glPushMatrix();
    glTranslated(0., 0.*size, -15.*size);
    glRotated(45.,0.,0.,1.); // Pour avoir un coté perpendiculaire a Ox0
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0., 1.414*size, 4, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 1.414*size, 1.414*size, 30.0*size, 4, 1);
    glTranslated(0., 0., 30.*size);
    gluDisk(quadric, 0., 1.414*size, 4, 1);
    // Le tube
    selectMaterial(9);
    glPopMatrix();
    glColor3ubv(colGrisfonce) ;
    glTranslated(0., (10.+1.)*size, -30.*size);
    glShadeModel(GL_SMOOTH);
    gluQuadricNormals(quadric, GLU_SMOOTH);
    gluQuadricOrientation(quadric, GLU_SMOOTH);
    // L'extérieur
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluDisk(quadric, 0., 10.*size, NDIV, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    gluCylinder(quadric, 10.*size, 10.*size, 60.0*size, NDIV, 1);
    // L'intérieur
    glTranslated(0., 0., 1.*size);
    gluDisk(quadric, 0., 9.5*size, NDIV, 1);
    gluQuadricOrientation(quadric, GLU_INSIDE);
    gluCylinder(quadric, 9.5*size, 9.5*size, 59.0*size, NDIV, 1);
    gluQuadricOrientation(quadric, GLU_OUTSIDE);
    // Le rebord
    glTranslated(0., 0., 59.*size);
    gluDisk(quadric, 9.5*size, 10.*size, NDIV, 1);
    glPopMatrix();
    glEndList(); /* Fin Scope */



    gluDeleteQuadric(quadric) ;

    // Initialise La vue
    glClearColor(0.06, 0.05, 0.04, 1.0);
    glEnable(GL_DEPTH_TEST);

    AziLum = AziLumIni = 50.;
    SitLum = SitLumIni = 50.;

    for (int i=0; i< dimConf; i++) sigma[i] = 0.;
    nbpars = dimConf;
    return ;
}

// Dessin de la scene exemple decrite ci-avant.
void Azeq6::redraw(void)
{
    char texte[256];
    font.setPointSize(8); // Pour les textes 2D.
    // Mise en place lumiere local
    double a = AziLum*M_PI/180., s = SitLum*M_PI/180., cs = cos(s);
    GLfloat pos_lumiere1[]={(GLfloat) (cos(a)*cs), (GLfloat) (sin(a)*cs), (GLfloat) sin(s),0.f};
    glEnable(GL_LIGHTING);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, faible); // par defaut
    glEnable(GL_LIGHT1);
    glLightfv(GL_LIGHT1, GL_POSITION, pos_lumiere1); // On modifie la position par défaut de GL_LIGHT1
    glLightfv(GL_LIGHT1, GL_SPECULAR, puissante);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, puissante);


    // On trace la scène
    glCallList(Colonne);
    glRotated(sigma[0]-90,0.,0.,1.);
    glCallList(Etrier);
    glRotated(90-sigma[1],1.,0.,0.);
    glCallList(SupEq6);
    glRotated(-90-sigma[2],0.,0.,1.);
    glCallList(CorpsEq6);
    glTranslated(0., 0., (8.+5.+1.)*size);
    glRotated(-90+sigma[3],0.,1.,0.);
    glCallList(Scope);

    // Fin dessin - Commute mode écriture textes
    if (!flagInfos) return;
    glDisable(GL_DEPTH_TEST);
    glViewport(0, 0, wGL->width1, wGL->height1);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0.0, wGL->width1, 0.0, wGL->height1, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glDisable(GL_LIGHTING);
    glColor3d(1., 1.,1.); // On ecrit en blanc

    sprintf(texte,"Position Oeil/scene : azi(<->) =%+4.2lf  sit(^-v) =%+4.2lf"
            "      Position éclairage : azi(R-r) =%+4.2lf  sit(S-s) =%+4.2lf"
            , wGL->AziCam, wGL->SitCam, AziLum, SitLum);
    wGL->renderText (10, 15, texte, font) ;
    sprintf(texte,"Azimut Etier/nord =%+4.2lf   Latitude =%+4.2lf",
            sigma[0], sigma[1]);
    wGL->renderText (10, 30, texte, font) ;
    char strhms[16];
    sprintf(texte,"angle horaire =%+4.2lf (%s)   Déclinaison =%+4.2lf",
            sigma[2], deg2strhms(strhms, sigma[2]), sigma[3]);
    wGL->renderText (10, 45, texte, font) ;

}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation
void Azeq6::setConfig(double *data)
{
    for (int i = 0; i < dimConf; i++) sigma[i] = data[i];
    wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* Peut être utilisée ou suchargée.                                                 */
/* a,A      : rotation en angle horaire                                             */
/* d,D      : rotation en déclinaison                                               */
/* h,H      : rotation en hauteur (latitude)                                        */
/* z,Z      : rotation en azimut                                                    */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool Azeq6::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;

    if (!fgoto)
    {
        switch (key)
        {
        case Qt::Key_I :
            flagInfos = !flagInfos;
            ok = TRUE;
            break ;
        case Qt::Key_R :
            AziLum -= DeltaA*smaj ;
            if (AziLum > 180.) AziLum -= 360. ;
            if (AziLum < -180.) AziLum += 360. ;
            ok = TRUE;
            break ;
        case Qt::Key_S :
            SitLum -= DeltaA*smaj ;
            if (SitLum > 90.) SitLum = 90. ;
            if (SitLum < -90.) SitLum = -90. ;
            ok = TRUE;
            break ;
        case Qt::Key_Escape : /* Point de vue initial */
            AziLum = AziLumIni; SitLum = SitLumIni;
        case Qt::Key_0 : /* Raz coordonnées */
            for(int i = 0; i < 4; i++) sigma[i] = 0.;
            ok = TRUE;
            break;
        case Qt::Key_O : // Configuration home
            sigma[0] = 0. ;
            sigma[1] = 45. ; // Latitude 45.
            sigma[2] = -90. ; // (AH = -6h)
            sigma[3] = 90. ; // Dec = 90
            ok = TRUE;
            break ;
        case Qt::Key_H : // angle Horaire
            sigma[2] -= DeltaA*smaj ;
            ok = TRUE;
            break ;
        case Qt::Key_D : // declinaison
            sigma[3] -= DeltaA*smaj ;
            ok = TRUE;
            break ;
        case Qt::Key_L : // Latitude (site)
            sigma[1] -= DeltaA*smaj ;
            ok = TRUE;
            break ;
        case Qt::Key_A : // Azimut / nord
            sigma[0] -= DeltaA*smaj ;
            ok = TRUE;
            break ;
        case Qt::Key_G : /* GoTo */
            fgoto = true;
            timerGo->start(deltaT);
            ok = true;
            break;

        case Qt::Key_W : /* GoTo */
            goDialog->init();
            if (goDialog->exec())
            {
                for(int i=0; i<dimConf;i++) sigDep[i] = sigma[i];
                fgoto = true;
                flagEnd = true;
                timerGo->start(deltaT);
            }
            else
                fgoto = false;
            ok = true;
            break;
        }
        if(ok && !fgoto) wGL->updateGL();
    }
    else
    {
        switch (key)
        {
        case Qt::Key_0 : /* au centre */
            fgoto = false;
            for(int i = 0; i < 4; i++) sigma[i] = 0.;
            wGL->updateGL();
            ok = true;
            break;
        case Qt::Key_O : // Configuration home
            fgoto = false;
            sigma[0] = 0. ;
            sigma[1] = 45. ; // Latitude 45.
            sigma[2] = 90. ; // (AH = 6h)
            sigma[3] = 90. ; // Dec = 90
            wGL->updateGL();
            ok = TRUE;
            break ;

        case Qt::Key_Space : /* pause */
            if (fpause) {fpause = false; timerGo->stop(); }
            else {fpause = true; timerGo->start(deltaT); }
            ok = true;
            break;
        }
    }
    return ok;
}

// Slot appele par le timeout du timerGo.
void Azeq6::gotoEnd(void)
{
    bool flagfin = true;
    double  omegT;
    double *sigBut = flagEnd ? sigEnd : sigDep;
    omegT = (20.)*deltaT/1000. ; // vitesse rotation = 20°/seconde
    // Génération d'un but intermédiaire entre sigma et sigbut en progressant de
    // omegT vers le sigbut. flagfin est mis à faux si un écart existe.
    for (int i = 0; i < dimConf; i++)
    {
        double dsig = sigBut[i] - sigma[i];
        if (dsig != 0.)
        {
            flagfin = false;
            if (dsig > 0.)
            {
                sigma[i] += omegT;
                if (sigma[i] > sigBut[i]) sigma[i] = sigBut[i];
            }
            else
            {
                sigma[i] -= omegT;
                if (sigma[i] < sigBut[i]) sigma[i] = sigBut[i];
            }
        }
    }
    if (flagfin)
    {
        timerGo->stop();
        fgoto = false;
        flagEnd = !flagEnd;
    }
    else wGL->updateGL();
}
