#include "caisse.h"
#include <GL/glu.h>
//#include <stdio.h>
//#include <string.h>
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)

#include "mllglutil.h"

Caisse::Caisse()
{
    dimConf = 6;
    delete labelHelp;
    labelHelp = new QLabel(
            tr(
                    "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p(minus) pour projection orthogonale</li>"
                    "<li>P(majus) pour perspective</li></ul>"
                    "<h5>Mouvements observateur :</h5>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h5>Mouvements maquette :</h5>"
                    "<ul><li>0,o,O: Retour origine</li>"
                    "<li>x, X : +/- Translation/Force en x</li>"
                    "<li>y, Y : +/- Translation/Force en y</li>"
                    "<li>z, Z : +/- Translation/Force en z</li>"
                    "<li>r, R : +/- Rotation/Moment en roulis(x)</li>"
                    "<li>t, T : +/- Rotation/Moment en tangage(y)</li>"
                    "<li>l, L : +/- Totation/Moment en lacet (z)</li></ul>"
                    ), 0);
    labelHelp->setWindowTitle("Aide Visu Caisse");
}

// Exemple avec un terrain plan fixe et un parallélépipède mobile
void Caisse::create()
{
    GLUquadricObj *quadric;

    glClearColor(0.85, 0.95, 0.95, 1.0);
    glShadeModel(GL_FLAT);
    glEnable(GL_DEPTH_TEST);

    /* Création objets*/
    quadric = gluNewQuadric();/* Nouvel objet quadric générique */
    gluQuadricTexture(quadric, GL_FALSE);/* qui ne sera pas éventuellemnt texturé */

    /* Construction Terrain (Rterrain avec z est vers le haut) */
    DL_Terrain = glGenLists(1);
    glNewList(DL_Terrain, GL_COMPILE);
    glColor3d(0.3,0.4,0.5);
    glPushMatrix(); /* Push repere base */
    glRotatef(45.,0.,0.,1.); // Pour avoir un coté perpendiculaire a Ox0
    gluDisk(quadric, 0.0, 4.0,4,1);
    glPopMatrix(); /* Retour repere base */
    glEndList();/* fin DL_Terrain */

    /* Construction Engin */
    DL_Engin = glGenLists(1);/* Un numéro de DisplayList */
    glNewList(DL_Engin, GL_COMPILE);/* Définition sans execution */
    glColor3d(0.7,0.1,0.1);
    glPushMatrix(); /* Push repere base */
    glScalef(1.,3.,0.5);
    glRotatef(45.,0.,0.,1.); // Pour avoir un coté perpendiculaire a Ox0
    gluDisk(quadric, 0.0, 0.5, 4, 1);
    gluCylinder(quadric, 0.5, 0.5, 1.0, 4, 1);
    glTranslatef(0.,-1.,0.);
    glPopMatrix(); /* Retour repere base */
    glEndList();/* fin de  DL_Engin */

    gluDeleteQuadric(quadric);

    /* Initialisation des positions  */
    OA[0] = -1.;  //X
    OA[1] = 0.;   //Y
    OA[2] = 0.;   //Z
    Carol[0] = Carol[1] = Carol[2] = 0.;
    poscarNok = true;
    nbpars = dimConf; // Pour setConfig
    return ; // Nombre de parametres de configuration
}

// Exemple de dessin de la scene exemple decrite ci-avant.
void Caisse::redraw(void)
{
    GLfloat pos_lumiere1[]={1.,0.,1.,0.}; /* selon bissectrice XZ */
    mlSetLight(GL_LIGHT0, pos_lumiere1);

    /* Les elements  */
    glCallList(DL_Terrain);

    glPushMatrix(); /* On push le repere base */
    /* Transformations pour situation dans repere engin */
    if(poscarNok)
    {
        double bid[3]; for(int i=0; i < 3; i++) bid[i] = Carol[i]*DEG2RAD;
        posEtcarol2mat44(sitOA44, OA, bid);
        poscarNok = false;
    }
    glMultMatrixd(sitOA44); /* On est dans le repere engin */
    glCallList(DL_Engin);
    glPopMatrix(); /* Retour repere base */
}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation. Ici un exemple basique
// de reception des 6 paramètres X,Y,Z,L,T,R
void Caisse::setConfig(double *data)
{
    int i;
        for (i = 0; i < 3; i++)
        {
            OA[i] = data[i];
            Carol[i] = data[i+3];
        }
        poscarNok = true;
        wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* Peut être utilisée ou suchargée.                                                 */
/* x,X,y,Y,z,Z : modifie la position de la maquette cad le vecteur OA               */
/* l,L      : modifie le cap cad Carol[0]                                           */
/* t,T      : modifie l'assiette cad Carol[1]                                       */
/* r,R      : modifie le roulis cad Carol[2]                                        */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool Caisse::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;
    double DeltaA = 2.5, DeltaL = 0.05;

    switch (key)
    {
    case Qt::Key_0 : /* au centre */
    case Qt::Key_O : /* au centre */
            OA[0]=-1.0;
            OA[1]=0.0;
            OA[2]=0.0;
            Carol[0]=0.0;
            Carol[1] =0.0;
            Carol[2]=0.0;
        ok = TRUE;
        break;
    case Qt::Key_X :
        OA[0] -= DeltaL*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_Y :
        OA[1] -= DeltaL*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_Z :
        OA[2] -= DeltaL*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_R : /* Roulis */
            Carol[2] -= DeltaA*smaj ;
            if (Carol[2]   >  180.0) Carol[2]   -= 360.0;
            if (Carol[2]   < -180.0) Carol[2]   += 360.0;
        ok = TRUE;
        break ;
    case Qt::Key_T : /* Assiette */
            Carol[1] -= DeltaA*smaj ;
            if (Carol[1] < -180.0) Carol[1] += 360. ;
            if (Carol[1] >  180.0) Carol[1] -= 360. ;
        ok = TRUE;
        break ;
    case Qt::Key_L : /* Cap */
            Carol[0] -= DeltaA*smaj ;
            if (Carol[0]      <    0.0) Carol[0]      += 360.0;
            if (Carol[0]      >  360.0) Carol[0]      -= 360.0;
        ok = TRUE;
        break ;
    }
    poscarNok = ok;
    if (ok) wGL->updateGL();
    return ok;
}
