#include "glterre.h"
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)

//#include "textbmap.h"
#include "../grafutils/qtglutils.h"
#include "mllglutil.h"

GlTerre::GlTerre(char *_filename)
{
    filename = _filename;
    labelHelp->setWindowTitle("Aide Visu Terre");
}

void GlTerre::create()
{
    glClearColor(0.85, 0.95, 0.95, 1.0);

    GLUquadricObj *qTerre = gluNewQuadric();
 //   gluQuadricDrawStyle(qTerre, GLU_FILL); // inutile car par défaut
    gluQuadricNormals(qTerre, GLU_NONE); // Pas de normales a générer
    
	// On charge la texture 
//    GLuint texture_earth = TextureLoad("../visusim/glterre/EARTH.BMP", GL_FALSE, GL_NEAREST, GL_NEAREST, GL_REPEAT);
    GLuint texture_earth = loadTexture(filename, GL_NEAREST, GL_NEAREST, GL_REPEAT);

    DL_Sphere = glGenLists(1);
    glNewList(DL_Sphere, GL_COMPILE); 
    // On associe la texture à la quadrique 	
    glBindTexture(GL_TEXTURE_2D, texture_earth);
    gluQuadricTexture(qTerre, GLU_TRUE); // On asssocier la texture
	
    glEnable(GL_TEXTURE_2D);
    gluSphere(qTerre, 2.0f, 25, 25);
    glDisable(GL_TEXTURE_2D); // Important pour les autres objets
    glEndList();

    gluDeleteQuadric(qTerre);
    OA[0] = 0.;  //X
    OA[1] = 0.;  //Y
    OA[2] = 0.;  //Z
    Carol[0] = Carol[1] = Carol[2] = 0.;
    poscarNok = true;

}

// Exemple de dessin de la scene exemple decrite ci-avant.
void GlTerre::redraw(void)
{
    GLfloat pos_lumiere1[]={1.,0.,1.,0.}; /* selon bissectrice XZ */
    mlSetLight(GL_LIGHT0, pos_lumiere1);
    /* Les elements  */
    /* Transformations pour situation dans repere enginn */
    if(poscarNok)
    {
        double bid[3]; for(int i=0; i < 3; i++) bid[i] = Carol[i]*DEG2RAD;
        posEtcarol2mat44(sitOA44, OA, bid);
        poscarNok = false;
    }
    glMultMatrixd(sitOA44); /* On est dans le repere engin */
    glCallList(DL_Sphere);
}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation. Ici un exemple basique
// de reception des 6 paramètres X,Y,Z,L,T,R
void GlTerre::setConfig(double *data)
{
//        wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* Peut être utilisée ou suchargée.                                                 */
/* x,X,y,Y,z,Z : modifie la position de la maquette cad le vecteur OA               */
/* l,L      : modifie le cap cad Carol[0]                                           */
/* t,T      : modifie l'assiette cad Carol[1]                                       */
/* r,R      : modifie le roulis cad Carol[2]                                        */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool GlTerre::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;
    double DeltaA = 2.5, DeltaL = 0.05;

    switch (key)
    {
    case Qt::Key_0 : /* au centre */
    case Qt::Key_O : /* au centre */
            OA[0]=-1.0;
            OA[1]=0.0;
            OA[2]=0.0;
            Carol[0]=0.0;
            Carol[1] =0.0;
            Carol[2]=0.0;
            poscarNok = true;
        ok = TRUE;
        break;
    case Qt::Key_X :
        OA[0] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_Y :
        OA[1] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_Z :
        OA[2] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_R : /* Roulis */
            Carol[2] -= DeltaA*smaj ;
            if (Carol[2]   >  180.0) Carol[2]   -= 360.0;
            if (Carol[2]   < -180.0) Carol[2]   += 360.0;
            poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_T : /* Assiette */
            Carol[1] -= DeltaA*smaj ;
            if (Carol[1] < -180.0) Carol[1] += 360. ;
            if (Carol[1] >  180.0) Carol[1] -= 360. ;
            poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_L : /* Cap */
            Carol[0] -= DeltaA*smaj ;
            if (Carol[0]      <    0.0) Carol[0]      += 360.0;
            if (Carol[0]      >  360.0) Carol[0]      -= 360.0;
            poscarNok = true;
        ok = TRUE;
        break ;
    }
    if (ok) wGL->updateGL();
    return ok;
}
