#include "mllglutil.h"
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)


/* Transformation dist, azi, sit -> xyz  */
void majsol(GLfloat *xyz, double dist, double azi, double sit)
{
    double p ;
    p = dist*cos(sit);
    xyz[0] = (GLfloat) (p*cos(azi));
    xyz[1] = (GLfloat) (p*sin(azi));
    xyz[2] = (GLfloat) (dist*sin(sit));
    xyz[3] = 1.f;
}

/*-------------------------------------------------------------------------------
    Active la lumiere numero GL_LIGHT# en position pos_lumiere[4 GLfloats]
--------------------------------------------------------------------------------*/
void mlSetLight(GLuint numLight, GLfloat *pos_lumiere)
{
    GLuint facemater, modemater;
    glEnable(GL_COLOR_MATERIAL); /* activation du coloriage simple (color tracking) */
    facemater = GL_FRONT_AND_BACK; /* à choisir parmi GL_FRONT, GL_BACK, GL_FRONT_AND_BACK */
    modemater = GL_AMBIENT_AND_DIFFUSE; /* à choisir parmi
        GL_AMBIENT  : couleur ambiante baignant la scene
        GL_DIFFUSE  : couleur emise par une source qui est réfléchie dans toutes les directions
        GL_SPECULAR : couleur emise par une source dont la reflection depend de la position de l'observateur
        GL_AMBIENT_AND_DIFFUSE */
    glColorMaterial(facemater, modemater);
    glEnable(GL_LIGHTING);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE); /* Les 2 faces sont traitées */
    glEnable(numLight);
    glLightfv(numLight, GL_POSITION, pos_lumiere);
}

/*---------------------------------------------------------------------------*/
/* Utilitaire conversion xyz, cap, assiette, roulis -> Situation OpenGL m44  */
/*---------------------------------------------------------------------------*/
double *posEtcarol2mat44(double *m44, double *pos, double *carol)
{
double cc, sc, ca, sa, cr, sr ;
double sasr, sacr ;

    cc = cos (carol[0]) ; ca = cos (carol[1]) ; cr = cos (carol[2]) ;
    sc = sin (carol[0]) ; sa = sin (carol[1]) ; sr = sin (carol[2]) ;
    sasr = sa * sr ; sacr = sa * cr ;

    m44[0] = cc*ca ; m44[4] = cc*sasr - sc*cr ;  m44[8]  = cc*sacr + sc*sr ; m44[12] = pos[0] ;
    m44[1] = sc*ca ; m44[5] = sc*sasr + cc*cr ;  m44[9]  = sc*sacr - cc*sr ; m44[13] = pos[1] ;
    m44[2] =   -sa ; m44[6] =    ca*sr        ;  m44[10] =    ca*cr        ; m44[14] = pos[2] ;
    m44[3] = 0.    ; m44[7] = 0.              ;  m44[11] = 0.              ; m44[15] = 1. ;
    return m44 ;
}

/*----------------------------------------------------------------------------------*/
/* Utilitaire conversion xyz et m33 -> Situation OpenGL  m44                        */
/*----------------------------------------------------------------------------------*/
double *posEtmat2GLmat44(double *m44, double *pos, double *mat)
{
    m44[0] = mat[0] ; m44[4] = mat[3] ;  m44[8]  = mat[ 6] ; m44[12] = pos[0] ;
    m44[1] = mat[1] ; m44[5] = mat[4] ;  m44[9]  = mat[ 7] ; m44[13] = pos[1] ;
    m44[2] = mat[2] ; m44[6] = mat[5] ;  m44[10] = mat[ 8] ; m44[14] = pos[2] ;
    m44[3] = 0.     ; m44[7] = 0.     ;  m44[11] = 0.      ; m44[15] = 1. ;
    return m44 ;
}
