#include "pleiadegl.h"
#include "mllglutil.h"

//#include <stdio.h>
//#include <string.h>
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)


GlPleiade::GlPleiade()
{
    delete labelHelp;
    labelHelp = new QLabel(
            tr(
                    "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p(minus) pour projection orthogonale</li>"
                    "<li>P(majus) pour perspective</li></ul>"
                    "<h5>Mouvements observateur :</h5>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h5>Mouvements maquette :</h5>"
                    "<ul><li>0,o,O: Retour origine</li>"
                    "<li>x, X : +/- Translation/Force en x</li>"
                    "<li>y, Y : +/- Translation/Force en y</li>"
                    "<li>z, Z : +/- Translation/Force en z</li>"
                    "<li>r, R : +/- Rotation/Moment en roulis(x)</li>"
                    "<li>t, T : +/- Rotation/Moment en tangage(y)</li>"
                    "<li>l, L : +/- Totation/Moment en lacet (z)</li></ul>"
                    ), 0);
    labelHelp->setWindowTitle("Aide Visu GlPleiade");
}

extern "C" { GLuint glNewPleiade(GLfloat size);}

void GlPleiade::create()
{
    DL_sat = glNewPleiade(0.5f);
    ///--- Ini parametres scene  ---///
    OA[0] = 0.;   //X
    OA[1] = 0.;   //Y
    OA[2] = 0.;   //Z
    Carol[0] = Carol[1] = Carol[2] = 0.;
    poscarNok = true;
    AziSol = 90.;
    SitSol = 60.;
    DistSol = 150.;
    majsol(PosiSol, DistSol, AziSol*DEG2RAD, SitSol*DEG2RAD);

    glClearColor(0.85, 0.95, 0.95, 1.0);
    glEnable(GL_DEPTH_TEST);
    return ;
}

// Exemple de dessin de la scene exemple decrite ci-avant.
void GlPleiade::redraw(void)
{
    /**********************************/
    /* L'éclairage                    */
    /**********************************/
    mlSetLight(GL_LIGHT0, PosiSol);

    /*----------------------
      On dessine la scene
    ----------------------*/
    glPushMatrix(); /* On push le repere base */
    /* Transformations pour situation dans repere enginn */
    if(poscarNok)
    {
        double bid[3]; for(int i=0; i < 3; i++) bid[i] = Carol[i]*DEG2RAD;
        posEtcarol2mat44(sitOA44, OA, bid);
        poscarNok = false;
    }
    glMultMatrixd(sitOA44); /* On est dans le repere engin */
        glCallList(DL_sat);
    glPopMatrix(); /* Retour repere base */

    return;
}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation. Ici un exemple basique
// de reception des 6 paramètres X,Y,Z,L,T,R
void GlPleiade::setConfig(double *data)
{
    wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* Peut être utilisée ou suchargée.                                                 */
/* x,X,y,Y,z,Z : modifie la position de la maquette cad le vecteur OA               */
/* l,L      : modifie le cap cad Carol[0]                                           */
/* t,T      : modifie l'assiette cad Carol[1]                                       */
/* r,R      : modifie le roulis cad Carol[2]                                        */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool GlPleiade::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;
    double DeltaA = 2.5, DeltaL = 0.05;

    switch (key)
    {
    case Qt::Key_0 : /* au centre */
    case Qt::Key_O : /* au centre */
            OA[0]=0.0;
            OA[1]=0.0;
            OA[2]=0.0;
            Carol[0]=0.0;
            Carol[1] =0.0;
            Carol[2]=0.0;
            poscarNok = true;
        ok = TRUE;
        break;
    case Qt::Key_X :
        OA[0] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_Y :
        OA[1] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_Z :
        OA[2] -= DeltaL*smaj ;
        poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_R : /* Roulis */
            Carol[2] -= DeltaA*smaj ;
            if (Carol[2]   >  180.0) Carol[2]   -= 360.0;
            if (Carol[2]   < -180.0) Carol[2]   += 360.0;
            poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_T : /* Assiette */
            Carol[1] -= DeltaA*smaj ;
            if (Carol[1] < -180.0) Carol[1] += 360. ;
            if (Carol[1] >  180.0) Carol[1] -= 360. ;
            poscarNok = true;
        ok = TRUE;
        break ;
    case Qt::Key_L : /* Cap */
            Carol[0] -= DeltaA*smaj ;
            if (Carol[0]      <    0.0) Carol[0]      += 360.0;
            if (Carol[0]      >  360.0) Carol[0]      -= 360.0;
            poscarNok = true;
        ok = TRUE;
        break ;
    }
    if (ok) wGL->updateGL();
    return ok;
}
