#include "georobot.h"
#include <float.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)


///////////////////////////////////////////////////////////////////////////////////////////
//   UTILITAIRES GEO ET ROBOT MLL
///////////////////////////////////////////////////////////////////////////////////////////
//Eventuellement a supprimer et remplacer par votre propre routine (utilise par solvlu)
static void mllPrintErreur(char *str){fprintf(stderr,"%s\n", str);}

static double longv3(double* a) /* FROM sp3.c */
{
    return(sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]));
}
static double lunitv3(double* a, double* b) /* FROM sp3.c */
{
    double l ;
    l = sqrt(b[0]*b[0]+b[1]*b[1]+b[2]*b[2]);
    if (l == 0.)
    {
        a[0]=1.; a[1]=0.; a[2]=0.;
    }
    else
    {
        a[0] = b[0]/l;
        a[1] = b[1]/l;
        a[2] = b[2]/l;
    }
    return(l);
}
static double *setv3(double* u, double s) /* FROM sp3.c */
{
    u[0] = s ;
    u[1] = s ;
    u[2] = s ;
    return u;
}
static double *cpyv3(double* u, double* v) /* FROM sp3.c */
{
    u[0] = v[0] ;
    u[1] = v[1] ;
    u[2] = v[2] ;
    return u;
}
static double *scav3(double* u, double s, double* v) /* FROM sp3.c */
{
    u[0] = s * v[0] ;
    u[1] = s * v[1] ;
    u[2] = s * v[2] ;
    return u;
}
static double *addv3(double* u, double* v, double* w) /* FROM sp3.c */
{
    u[0] = v[0] + w[0] ;
    u[1] = v[1] + w[1] ;
    u[2] = v[2] + w[2] ;
    return u;
}
static double *subv3(double* u, double* v, double* w) /* FROM sp3.c */
{
    u[0] = v[0] - w[0] ;
    u[1] = v[1] - w[1] ;
    u[2] = v[2] - w[2] ;
    return u;
}
static double *cumv3(double* u, double* v,double a, double* w) /* FROM sp3.c */
{
    u[0] = v[0] + a*w[0] ;
    u[1] = v[1] + a*w[1] ;
    u[2] = v[2] + a*w[2] ;
    return u;
}
static double *crossv3(double* a, double* b, double* c) /* FROM sp3.c */
{
    double p, q ;
    p  = b[1] * c[2] - b[2] * c[1];
    q  = b[2] * c[0] - b[0] * c[2];
    a[2] = b[0] * c[1] - b[1] * c[0];
    a[0] = p ;
    a[1] = q ;
    return a;
}
static double *eyem3(double *a) /* FROM sp3.c */
{
    a[0] = a[4] = a[8] = 1. ;
    a[1] = a[2] = a[3] = a[5] = a[6] = a[7] = 0. ;
    return a ;
}
static double *m3v3(double* w, double* a, double* v) /* FROM sp3.c */
{
    double p, q ;
    p  = a[0] * v[0] + a[3] * v[1] + a[6] * v[2];
    q  = a[1] * v[0] + a[4] * v[1] + a[7] * v[2];
    w[2] = a[2] * v[0] + a[5] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
    return w;
}
static double *tm3v3(double* w, double* a, double* v) /* FROM sp3.c */
{
    double p, q ;
    p  = a[0] * v[0] + a[1] * v[1] + a[2] * v[2];
    q  = a[3] * v[0] + a[4] * v[1] + a[5] * v[2];
    w[2] = a[6] * v[0] + a[7] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
    return w;
}
double *m3m3(double* a,double* b, double* c) /* FROM sp3.c */
{
    register int i ;
    double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[1] + b[6] * c[2];
    d[1] = b[1] * c[0] + b[4] * c[1] + b[7] * c[2];
    d[2] = b[2] * c[0] + b[5] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[3] * c[4] + b[6] * c[5];
    d[4] = b[1] * c[3] + b[4] * c[4] + b[7] * c[5];
    d[5] = b[2] * c[3] + b[5] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[3] * c[7] + b[6] * c[8];
    d[7] = b[1] * c[6] + b[4] * c[7] + b[7] * c[8];
    a[8] = b[2] * c[6] + b[5] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
    return a;
}
static double *m3tm3(double* a, double* b, double* c) /* FROM sp3.c */
{
    register int i ;
    double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[3] + b[6] * c[6];
    d[1] = b[1] * c[0] + b[4] * c[3] + b[7] * c[6];
    d[2] = b[2] * c[0] + b[5] * c[3] + b[8] * c[6];

    d[3] = b[0] * c[1] + b[3] * c[4] + b[6] * c[7];
    d[4] = b[1] * c[1] + b[4] * c[4] + b[7] * c[7];
    d[5] = b[2] * c[1] + b[5] * c[4] + b[8] * c[7];

    d[6] = b[0] * c[2] + b[3] * c[5] + b[6] * c[8];
    d[7] = b[1] * c[2] + b[4] * c[5] + b[7] * c[8];
    a[8] = b[2] * c[2] + b[5] * c[5] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
    return a;
}
static double *tm3m3( double* a,double* b, double* c) /* FROM sp3.c */
{
    register int i;
    double d[8] ;
    d[0] = b[0] * c[0] + b[1] * c[1] + b[2] * c[2];
    d[1] = b[3] * c[0] + b[4] * c[1] + b[5] * c[2];
    d[2] = b[6] * c[0] + b[7] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[1] * c[4] + b[2] * c[5];
    d[4] = b[3] * c[3] + b[4] * c[4] + b[5] * c[5];
    d[5] = b[6] * c[3] + b[7] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[1] * c[7] + b[2] * c[8];
    d[7] = b[3] * c[6] + b[4] * c[7] + b[5] * c[8];
    a[8] = b[6] * c[6] + b[7] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
    return a;
}
//----------------------------------------------------------------------------
double *mrotx(double *m, double a) /* FROM rotation.c */
{
    double s = sin(a), c = cos(a);
    m[0] = 1.; m[3] = 0.; m[6] = 0.;
    m[1] = 0.; m[4] = c ; m[7] =-s ;
    m[2] = 0.; m[5] = s ; m[8] = c ;
    return m ;
}
double *mroty(double *m, double a) /* FROM rotation.c */
{
    double s = sin(a), c = cos(a);
    m[0] = c ; m[3] = 0.; m[6] = s ;
    m[1] = 0.; m[4] = 1.; m[7] = 0.;
    m[2] =-s ; m[5] = 0.; m[8] = c ;
    return m ;
}
double *mrotz(double *m, double a) /* FROM rotation.c */
{
    double s = sin(a), c = cos(a);
    m[0] = c ; m[3] =-s ; m[6] = 0.;
    m[1] = s ; m[4] = c ; m[7] = 0.;
    m[2] = 0.; m[5] = 0.; m[8] = 1.;
    return m ;
}
static double *av3tomrot(double* m, double a, double* v) /* FROM rotation.c */
{
    double wx, wy, wz;
    double k;

    wx = v[0];
    wy = v[1];
    wz = v[2];
    k = 1.-cos(a);
    m[0] = 1. + k * (wx * wx - 1.);
    m[4] = 1. + k * (wy * wy - 1.);
    m[8] = 1. + k * (wz * wz - 1.);
    m[1] = k * wx * wy;
    m[2] = k * wz * wx;
    m[5] = k * wy * wz;
    k = sin(a);
    wx = k * wx;
    wy = k * wy;
    wz = k * wz;
    m[7] = m[5] - wx;
    m[5] += wx;
    m[6] = m[2] + wy;
    m[2] -= wy;
    m[3] = m[1] - wz;
    m[1] += wz;
    return m;
}
static double *porpor(double* a, double* b, double* c) /* FROM rotation.c */
{
    double d[3] ;
    m3v3(d, b+3, c);
    m3m3(a+3, b+3, c+3);
    addv3(a, b, d) ;
    return a;
}
static double *porpori(double* a, double* b, double* c) /* FROM rotation.c */
{
    double d[3] ;
    m3tm3(a+3, b+3 ,c+3) ;
    m3v3(d, a+3, c) ;
    subv3(a, b, d) ;
    return a;
}
double *mrot2ltr (double *ltr, double *A) /* FROM rotation.c */
{
    double l, t, r, som,dif,pis2 ;

    pis2 = M_PI/2. ;
    t = asin( A[5] ) ;
    if ( fabs(t) < pis2 - 1.E-6)
    {
        l = atan2( -A[3] , A[4] ) ;
        r = atan2( -A[2] , A[8] ) ;
    }
    else
    {
        if ( t > 0.)
        {
            som = atan2( A[1],A[0] ) ;
            dif = ltr[0] - ltr[2] ;
        }
        else
        {
            dif = atan2( A[1],A[0] ) ;
            som = ltr[0] + ltr[2] ;
        }
        l = (som + dif)/2. ;
        r = (som - dif)/2. ;
    }

    if(fabs(ltr[1]) > pis2)
    {
        if(l < 0.) l = M_PI + l ; else l = -M_PI + l ;
        if(t > 0.) t = M_PI - t ; else t = -M_PI - t ;
        if(r < 0.) r = M_PI + r ; else r = -M_PI + r ;
    }
    ltr[0] = l ;
    ltr[1] = t ;
    ltr[2] = r ;

    return ltr;
}
//------------------------------------------------------------------------
double *setvec (double* u, double s, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = s;
    return u;
}
double *scavec (double *u, double a, double *v, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = a * v[n] ;
    return u;
}
static double *cpyvec (double* u, double* v, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = v[n];
    return u;
}
static double *absvec (double* u, double* v, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = fabs(v[n]);
    return u;
}
static double *addvec (double* u, double* v, double* w, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = v[n] + w[n];
    return u;
}
//static double *subvec (double* u, double* v, double* w, int n) /* FROM spn.c */
//{
//    if (n > 0) while (n--) u[n] = v[n] - w[n];
//    return u;
//}
static double dotvec (double* u, double* v, int n) /* FROM spn.c */
{
    double s ;
    s = 0. ;
    if (n > 0) while (n--) s += u[n] * v[n] ;
    return(s);
}
static int iamaxvec (double *u, int n) /* FROM spn.c */
{
    int i, id ;
    double dmax, a ;

    if (n <= 1) return (n-1) ;
    dmax = fabs(u[0]) ; id = 0 ;
    for (i = 1 ; i < n ; i++)
    { a = fabs(u[i]) ; if (a > dmax) {id = i ; dmax = a ; } }
    return id ;
}
static double xmaxvec (double* u, int n) /* FROM spn.c */
{
    double s = -DBL_MAX;
    if (n > 0) while (n--) if (u[n]>s) s = u[n];
    return s;
}
static double *divvec (double* u, double* v, double* w, int n) /* FROM spn.c */
{
    if (n > 0) while (n--) u[n] = v[n] / w[n];
    return u;
}
static double *mattmat (double *a, double *b, double *c, int m, int n, int p) /* FROM spn.c */
{
    int i, j, k, ij, ik, jk ;

    ij = -1 ;
    for (j = 0 ; j < p ; j++)
        for (i = 0 ; i < m ; i++)
        {
        a[++ij] = 0. ;
        for (k = 0, ik = i, jk = j ; k < n ; k++, ik += m, jk += p)
            a[ij] += b[ik]*c[jk] ;
    }
    return a;
}
static double *tmatmat (double *a, double *b, double *c, int m, int n, int p) /* FROM spn.c */
{
    int i, j, k, ij, ki, kj, kj0 ;

    for (j = 0, ij = 0, kj0 = 0 ; j < p ; j++, kj0 += n)
    {
        for (i = 0, ki = 0 ; i < m ; i++, ij++)
        {
            a[ij] = 0. ;
            for (k = 0, kj = kj0 ; k < n ; k++) a[ij] += b[ki++] * c[kj++] ;
        }
    }
    return a;
}
//--------------------------------------------------------------------------------
static void cum1vec (double *y, double a, double *x, int n) /* FROM linsolve.c */
{
    /* Cumulation de vecteurs */
    if (n > 0 && a != 0.) while (n--) y[n] += a * x[n] ;
}
static void sca1vec (double *x, double a, int n) /* FROM linsolve.c */
{ /* multiplie un vecteur par une constante */

    if (n > 0) while (n--) x[n] = a * x[n] ;
}
static double lufactor (double *a, int lda, int n, int *ipvt) /* FROM linsolve.c */
{
/*
     lufactor effectue la factorisation L U d'une matrice n x n
     extraite d'une matrice rangees par vecteurs de lda composantes.
     Les facteurs sont renvoyes dans A et les indices des pivots
     des lignes dans ipvt.
     Renvoie le determinant

     Adapte en C par MLL le 28/04/92 d'après
     LINPACK. 08/14/78 CLEVE MOLER, UNIVERSITY OF NEW MEXICO
     ARGONNE NATIONAL LAB.
     Utilise cum1vec, sca1vec
*/

double t, det=1. ;
int j, k, l, nm1, lk, kk, lj, kj, klda;
/* int info; */ /* indice fortran du 1er element nul de la diag */
int lda1 = 1 + lda ;

/*  Elimination de Gauss avec pivotage partiel */

/*    info = 0 ; */
    nm1 = n - 1 ;
    kk = 0 ;
    if (nm1 >= 1)
    {
        for (k = 0 , klda = 0 , kk = 0 ; k < nm1 ;
                         k++ , klda += lda, kk += lda1)
        {
        /*  Recherche l = indice du pivot */
            l = iamaxvec (&a[kk], n-k) + k ;
            ipvt[k] = l ;
        /* Si 0 la colonne est deja triangularisee */
            lk = l + klda ;
            if ( a[lk] != 0.)
            {
            if (l != k) { t = a[lk] ; a[lk] = a[kk] ; a[kk] = t; det = -det;}
            det *= a[kk];
            /* Calcul des multiplicateurs */
            t = -1./a[kk] ;
            sca1vec (&a[kk+1], t, nm1-k) ;
            /* Elimination des colonnes */
            lj = lk ; kj = kk ;
            for (j = k+1 ; j < n ; j++)
            {
                lj += lda ; kj += lda ; t= a[lj] ;
                if (l != k) { a[lj] = a[kj] ; a[kj] = t ; }
                cum1vec (&a[kj+1], t, &a[kk+1], nm1-k) ;
            }
            }
/*          else info = k+1 ; */
            else det = 0.;
        }
    }
    ipvt[nm1] = nm1 ;
    kk = nm1*lda1 ;
/*    if ( a[kk] == 0.) info = n ; */
    det *= a[kk];
    return det ;
}
static void lusolv (double *a, int lda, int n, int *ipvt, double *b, int trp) /* FROM linsolve.c */
{
/*
      Renvoie le determinant. Si 0 les calculs sont inacheves
      Adapte en C par MLL le 28/04/92 d'après :
      LINPACK. THIS VERSION DATED 08/14/78 .
      CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
*/
double t, diag ;
int k, kp1, klda, l, nm1 ;

nm1 = n - 1 ;
if (trp == 0 )
{
/*
         trp = 0 , resout  A * X = B
         resout d'abord  L*Y = B
*/
    if (nm1 > 0)
    {
        for (k = 0 , klda = 0 ; k < nm1 ; k++, klda += lda )
        {
            kp1 = k + 1 ;
            l = ipvt[k] ; t = b[l] ;
            if (l != k) { b[l] = b[k] ; b[k] = t ;}
            cum1vec (&b[kp1], t, &a[klda+kp1], n-k-1) ;
        }
    }
/*
         resout  U*X = Y
*/
    for (k = nm1 , klda = nm1 * lda ; k >= 0 ; k--, klda -= lda)
    {
        if ( (diag = a[k+klda]) == 0.) return ;
        b[k] /= diag ;
        t = - b[k] ;
        cum1vec (b, t , &a[klda], k) ;
    }
}
else
{
/*
     trp = 1, resout  A' * X = B
     resout d'abord   U' * Y = B
*/
    for (k = 0, klda = 0 ; k < n ; k++, klda += lda)
    {
        t = dotvec (&a[klda], b, k) ;
        if ( (diag = a[k+klda]) == 0.) return ;
        b[k] = ( b[k] - t ) / diag ;
    }
/*
    resout L' * X = Y
*/
    if (nm1 > 0)
    {
        for (k = n-2, klda = k * lda ; k >= 0 ; k-- , klda -= lda)
        {
            kp1 = k + 1 ;
            b[k] += dotvec (&a[klda+kp1], &b[kp1], nm1 - k) ;
            l = ipvt[k] ;
            if (l != k) {t = b[l]; b[l] = b[k] ; b[k] = t ;}
        }
    }
}
}
/*---------------------------------------------------------------------
                   /  Y = A  * X  (si trp = 0)  -> X = inv(A)  * Y
    Résolution de  |
                   \  Y = A' * X  (si trp = 1)  -> X = inv(A') * Y
    -------------------------------------------------------------------

    !!!!!!!!!!  ATTENTION : LA MATRICE A EST DETRUITE  !!!!!!!!!!!

    Entrees:
        double *XY      : Adresse des ns vecteurs Y donnés
                                                  remplacés par ns vecteurs X résultats
        double *A       : Adresse de A (qui sera détruite)
        int    n        : dimension de A (n x n)
        int    ns       : Nombre de colonnes de XY
        int    trp      : Flag de transposition de A

    Sortie :
        X est calculée
        Renvoie le determinant
 Si Det == 0, résolution impossible (peut être echec allocation ?)
----------------------------------------------------------------------*/
double solvlu (double *XY, double *A, int n, int ns, int trp)/* FROM linsolve.c */
{
double det ;

int *ipvt = (int *) malloc (n*sizeof(int)) ;
    if (ipvt == NULL){mllPrintErreur("Echec allocation dans solvlu !"); return 0. ;}
    if ( (det=lufactor (A, n, n, ipvt)) != 0)
    {
        while (ns--) lusolv (A, n, n, ipvt, &XY[n*ns], trp) ;
    }
    free(ipvt);
    return det ;
}
/*------------------------------------------------------------------
 Calcul modèle géométrique Dop du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2Dop(UnRobot *rob, double *qi, double *Dop)
{
    int i ;
    Por Dij;
    eyem3(Dop+3); setv3(Dop,0.) ;
    for (i = rob->nq-1 ; i >= 0; i--)
    {
        if (rob->ri[i]) /* Cas rotation */
        {
            cpyv3(Dij, rob->li[i]);
            av3tomrot(Dij+3, qi[i], rob->ui[i]);
        }
        else /* Cas translation */
        {
            cumv3(Dij, rob->li[i], qi[i], rob->li[i]);
            eyem3(Dij+3);
        }
        porpor(Dop,Dij,Dop) ;
    }
}
/*------------------------------------------------------------------
 Calcul modèle géométrique Dom du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2Dom(UnRobot *rob, double *qi, double *Dom)
{
    Por Dop;
    qi2Dop(rob, qi, Dop);
    porpor(Dom,Dop,rob->Dpm) ;
}
/*------------------------------------------------------------------
      Calcul modèle géométrique Dop et jacobienne J dans Rp
      du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2DopetJp(UnRobot *rob, double *qi, double *Dop, double *Jacp)
{
    int i ;
    double *Jv, *Jw;
    Por Dij;
    eyem3(Dop+3); setv3(Dop,0.) ;
    for (i = rob->nq-1 ; i >= 0; i--)
    {
        Jv = &Jacp[i*6] ; Jw = Jv+3 ;
        if (rob->ri[i]) /* Cas rotation */
        {
            cpyv3(Dij, rob->li[i]);
            av3tomrot(Dij+3, qi[i], rob->ui[i]);

            crossv3(Jv, rob->ui[i], Dop);
            cpyv3(Jw, rob->ui[i]);
        }
        else /* Cas translation */
        {
            cumv3(Dij, rob->li[i], qi[i], rob->li[i]);
            eyem3(Dij+3);

            cpyv3(Jv, rob->ui[i]);
            setv3(Jw, 0.);
        }
        tm3v3 (Jv, Dop+3, Jv) ;
        tm3v3 (Jw, Dop+3, Jw) ;
        porpor(Dop,Dij,Dop) ;
    }
}
/*------------------------------------------------------------------
      Calcul modèle géométrique Dom et jacobienne J dans Rm
          du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2DometJm(UnRobot *rob, double *qi, double *Dom, double *Jacm)
{
    int i;
    Por Dop;
    double *Cpm = rob->Dpm+3;
    double *Jv, *Jw;
    qi2DopetJp(rob, qi, Dop, Jacm);
    porpor(Dom,Dop,rob->Dpm) ;
    for (i = rob->nq-1 ; i >= 0; i--)
    {
        Jv = &Jacm[i*6] ; Jw = Jv+3 ;
        tm3v3(Jv,Cpm, Jv);
        tm3v3(Jw,Cpm, Jw);
    }
}

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dop
            dans la même nappe que le qi initial
                (en theorie, sauf franchissement accidentel point singulier)
     L'inversion est faite par itérations successives en utilisant la
     Jacobienne, avec :
     - maxiter : nombre maximal d'itérations,
     - epsang : epsilon de variation de configuration qu à partir de laquelle
       on considère que l'inversion est achevée (convergence terminée).
     - epsang : epsilon de situation à partir de laquelle on considère que
       l'inversion est achevée (convergence terminée).
     - dalim : variation d'attitude maximale autorisée par itération
     - dplim : variation de position maximale autorisée par itération
     - dqlim : variation de configuration maximale autorisée par itération
         Renvoi :
         = nombre d'itérations >= 0 si convergence
         = -1 si divergence
         = -2 si nombre max d'itérations atteint
         = -3 si singularité résolution
------------------------------------------------------------------------*/
int Dop2qi(UnRobot *rob, double *Dop, double *qi, int maxiter,
                  double epsang, double epspos, double dalim, double dplim, double *dqlim)
{
    double Jacp[6*NQMAX], JJ[36] ;
    Por Dcou;
    double Erpos[6], *Eratt = Erpos+3;
    double Delq[NQMAX], coefs[NQMAX], cmax ;
    Mat3 E;
    int k = 0, N = rob->nq ;
    double ang, dist;
    double oldang = 1.e+100, olddist = 1.e+100;

    if (N > NQMAX)
    {
        printf("NQMAX limité à 10. Recompiler pour augmenter sa valeur !");
        exit(1) ;
    }
    if (N < 6)
    {
        printf("L'inversion suppose au moins 6 axes !");
        exit(1) ;
    }
    do
    {
        double det;
        /* Situation courante */
        qi2DopetJp(rob, qi, Dcou, Jacp) ;
        /* Erreur de position */
        subv3(Erpos, Dop, Dcou);
        tm3v3(Erpos, Dop+3, Erpos) ; /* Dans Rp */
        dist = longv3(Erpos) ;
        /* Erreur d'orientation dans Rp */
        tm3m3(E, Dcou+3, Dop+3);
        Eratt[0] = (E[5] - E[7])/2. ;
        Eratt[1] = (E[6] - E[2])/2. ;
        Eratt[2] = (E[1] - E[3])/2. ;
        /* On confond ang avec sin(ang), saturé à 1 si cos(ang) < 0. */
        if( E[0]+E[4]+E[8] < 1.) { ang = 1. ; lunitv3(Eratt, Eratt); }
        /* Supprime car precision inutile .........
                else { ang = asin(lunitv3(Eratt, Eratt)); scav3(Eratt, ang, Eratt);} */
        else { ang = longv3(Eratt);}
        /* test si convergence */
        if (ang < epsang && dist < epspos) return k ;
        /* test si divergence */
        //if (ang > oldang || dist > olddist) return -1;
        oldang = ang ; olddist = dist;
        /* saturations homothétiques */
        if (dist > dplim) scav3(Erpos, dplim/dist, Erpos) ;
        if (ang > dalim) scav3(Eratt, dalim/ang, Eratt) ;
        if (N > 6) /* Cas redondant */
        {
            mattmat (JJ, Jacp, Jacp, 6, N, 6) ; /* J*J'*/
            det = solvlu (Erpos, JJ, 6, 1, 0) ;
            if (det==0.) return -3 ;
            tmatmat (Delq, Jacp, Erpos, N, 6, 1) ;
        }
        else if (N == 6) /* Cas standard */
        {
            cpyvec(Delq, Erpos, 6) ;
            det = solvlu (Delq, Jacp, 6, 1, 0) ;
            if (det==0.) return -3 ;
        }
        else /* Cas sous-déterminé */
        {
            tmatmat (Delq, Jacp, Erpos, N, 6, 1) ;
            tmatmat (JJ, Jacp, Jacp, N, 6, N) ; /* J'*J*/
            det = solvlu (Delq, JJ, N, 1, 0) ;
            if (det==0.) return -3 ;
        }
        /* Saturation homothétique du Deltaq */
        divvec(coefs,Delq,dqlim,N);
        cmax = xmaxvec(absvec(coefs, coefs,N),N) ;
        if (cmax > 1.) scavec(Delq,1./cmax,Delq,N);
        addvec (qi, qi, Delq, N) ;

        k++;
        if (k > maxiter) return -2 ;
    } while (1) ;
    return k; /* bidon */
}

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dom
            dans la même nappe que le qi initial
                (en theorie, sauf franchissement accidentel point singulier)
     L'inversion est faite par itérations successives en utilisant la
     Jacobienne, avec :
     - maxiter : nombre maximal d'itérations,
     - epsang : epsilon de variation de configuration qu à partir de laquelle
       on considère que l'inversion est achevée (convergence terminée).
     - epsang : epsilon de situation à partir de laquelle on considère que
       l'inversion est achevée (convergence terminée).
     - dalim : variation d'attitude maximale autorisée par itération
     - dplim : variation de position maximale autorisée par itération
     - dqlim : variation de configuration maximale autorisée par itération
         Renvoi :
         = nombre d'itérations >= 0 si convergence
         = -1 si divergence
         = -2 si nombre max d'itérations atteint
         = -3 si singularité résolution
------------------------------------------------------------------------*/
int Dom2qi(UnRobot *rob, double *Dom, double *qi, int maxiter,
                  double epsang, double epspos, double dalim, double dplim, double *dqlim)
{
    Por Dop;
    porpori(Dop,Dom,rob->Dpm);
    return Dop2qi(rob, Dop, qi, maxiter, epsang, epspos, dalim, dplim, dqlim);
}

///////////////////////////////////////////////////////////////////////////////////////////
//  FIN UTILITAIRES GEO ET ROBOT MLL
///////////////////////////////////////////////////////////////////////////////////////////
UnRobot *newDataRv80()
{
    UnRobot *rob;
    int i, n, typ ;
    double x,y,z,azi,sit,csit;
    char DataRv80[6][16] = { {"1 0 0 1.5  0 90"},
                             {"1 0 0 0    0  0"},
                             {"1 0 1 0    0  0"},
                             {"1 0 1 0   90  0"},
                             {"1 0 0 0    0  0"},
                             {"1 0 0 0   90  0"}};

    i = sizeof(double) ;
    n = sizeof(Vec3);
    rob = (UnRobot *) malloc(sizeof(UnRobot)+6*sizeof(Vec3)+6*sizeof(Vec3)+6*sizeof(int));
    rob->ui = (Vec3 *) &(rob[1]);
    rob->li = &(rob->ui[6]);
    rob->ri = (int *) &(rob->li[6]);
    rob->nq = 6 ;
    //    rob->ri = (int *) malloc(rob->nq*sizeof(int));;
    //    rob->ui = (Vec3 *) malloc(rob->nq*sizeof(Vec3));
    //    rob->li = (Vec3 *) malloc(rob->nq*sizeof(Vec3));
    for( n = 0 ; n < 6 ; n++)
    {
        sscanf(DataRv80[n],"%d %lf %lf %lf %lf %lf",&typ, &x, &y, &z, &azi, &sit);
        sit *= M_PI/180. ; azi *= M_PI/180.;
        rob->ri[n] = typ ;
        rob->li[n][0] = x ;
        rob->li[n][1] = y ;
        rob->li[n][2] = z ;
        csit = cos(sit) ;
        rob->ui[n][0] = cos(azi)*csit ;
        rob->ui[n][1] = sin(azi)*csit ;
        rob->ui[n][2] = sin(sit) ;
    }
    rob->Dpm[0]=rob->Dpm[2]=0.; rob->Dpm[1]=0.371; eyem3(rob->Dpm+3);
    return rob;
}

