#ifndef GEOROBOT_H
#define GEOROBOT_H

#  ifdef __cplusplus
extern "C" {
#  endif


// Utilitaires geo pur
double *setvec (double* u, double s, int n);
double *scavec (double *u, double a, double *v, int n);
double *m3m3(double* a,double* b, double* c);
double *mrotx(double *m, double a);
double *mroty(double *m, double a);
double *mrotz(double *m, double a);
/*----------------------------------------------------------------------
        Extraction des angles l,t,r (/Z0,/X1,/Y2) d'une matrice de rotation
        --------------------------------------------------------------------

        Entrees:
                double *ltr	: Adresse des angles l,t,r  (radians)
                double *A	: Adresse de la matrice de rotation donné

        Sorties:
                angles ltr calculés (radians)
                Retour		: Adresse des angles l,t,r

    Remarque :
                Pour abs(t) > pi/2 - 1.E-6   conservation de la somme ou de la
                difference des valeurs ltr passées en référence.
----------------------------------------------------------------------*/
double *mrot2ltr (double *ltr, double *A);

#define NQMAX 10
typedef double Vec3[3] ;
typedef double Mat3[9] ;
typedef double Por[12] ;

typedef struct
{
        int nq;
        int *ri;
        Vec3 *ui;
        Vec3 *li;
        Por Dpm ;
} UnRobot ;


/*------------------------------------------------------------------
      Calcul modèle géométrique Dop et jacobienne J dans Rp
          du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2DopetJp(UnRobot *rob, double *qi, double *Dop, double *Jacp);

/*------------------------------------------------------------------
      Calcul modèle géométrique Dom et jacobienne J dans Rm
          du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2DopetJp(UnRobot *rob, double *qi, double *Dom, double *Jacm);

/*------------------------------------------------------------------
Calcul modèle géométrique Dop du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2Dop(UnRobot *rob, double *qi, double *Dop);

/*------------------------------------------------------------------
 Calcul modèle géométrique Dom du robot rob, pour la configuration qi
--------------------------------------------------------------------*/
void qi2Dom(UnRobot *rob, double *qi, double *Dom);

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dop
            dans la même nappe que le qi initial
                (en theorie, sauf franchissement accidentel point singulier)
     L'inversion est faite par itérations successives en utilisant la
     Jacobienne, avec :
     - maxiter : nombre maximal d'itérations,
     - epsang : epsilon de variation de configuration qu à partir de laquelle
       on considère que l'inversion est achevée (convergence terminée).
     - epsang : epsilon de situation à partir de laquelle on considère que
       l'inversion est achevée (convergence terminée).
     - dalim : variation d'attitude maximale autorisée par itération
     - dplim : variation de position maximale autorisée par itération
     - dqlim : variation de configuration maximale autorisée par itération
         Renvoi :
         = nombre d'itérations >= 0 si convergence
         = -1 si divergence
         = -2 si nombre max d'itérations atteint
         = -3 si singularité résolution
------------------------------------------------------------------------*/
int Dop2qi(UnRobot *rob, double *Dop, double *qi, int maxiter,
                  double epsang, double epspos, double dalim, double dplim, double *dqlim);

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dom
            dans la même nappe que le qi initial
                (en theorie, sauf franchissement accidentel point singulier)
     L'inversion est faite par itérations successives en utilisant la
     Jacobienne, avec :
     - maxiter : nombre maximal d'itérations,
     - epsang : epsilon de variation de configuration qu à partir de laquelle
       on considère que l'inversion est achevée (convergence terminée).
     - epsang : epsilon de situation à partir de laquelle on considère que
       l'inversion est achevée (convergence terminée).
     - dalim : variation d'attitude maximale autorisée par itération
     - dplim : variation de position maximale autorisée par itération
     - dqlim : variation de configuration maximale autorisée par itération
         Renvoi :
         = nombre d'itérations >= 0 si convergence
         = -1 si divergence
         = -2 si nombre max d'itérations atteint
         = -3 si singularité résolution
------------------------------------------------------------------------*/
int Dom2qi(UnRobot *rob, double *Dom, double *qi, int maxiter,
                  double epsang, double epspos, double dalim, double dplim, double *dqlim);

/*---------------------------------------------------------------------------
  Creation d'une structure de donnees pour les modeles geometriques et
  cinematique d'un robot RV80
  Pour la détruire faire simplement : free(UnRobot *);
-----------------------------------------------------------------------------*/
UnRobot *newDataRv80();

#  ifdef __cplusplus
}
#  endif


#endif // GEOROBOT_H
