#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)

///////////////////////////////////////////////////////////////////////////////////
//  UTILITAIRES MLL DE DESSIN OPENGL
///////////////////////////////////////////////////////////////////////////////////
typedef struct MLLquadricObj {
    GLenum	DrawStyle;		/* GLU_FILL, LINE, SILHOUETTE, or POINT */
    GLenum Orientation;		/* GLU_INSIDE or GLU_OUTSIDE */
    GLboolean TextureFlag;	/* Generate texture coords? */
    GLenum Normals;			/* GLU_NONE, GLU_FLAT, or GLU_SMOOTH */
    void (*ErrorFunc)(GLenum err);	/* Error handler callback function */
} MLLquadricObj;

///* Texture coordinates if texture flag is set */
//#define TXTR_COORD(x,y)    if (qobj->TextureFlag) glTexCoord2f(x,y);

static MLLquadricObj *mllNewQuadric( void )
{
    MLLquadricObj *q;

    q = (MLLquadricObj *) malloc( sizeof(struct MLLquadricObj) );
    if (q) {
        q->DrawStyle = GLU_FILL;
        q->Orientation = GLU_OUTSIDE;
        q->TextureFlag = GL_FALSE;
        q->Normals = GLU_SMOOTH;
        q->ErrorFunc = NULL;
    }
    return q;
}


static void mllDeleteQuadric( MLLquadricObj *state )
{
    if (state) {
        free( (void *) state );
    }
}

/*---------------------------------------------------------------------------------
    Parallélépipède avec ou sans fond et couvercle.
        Les cotés sont parallèles aux plans du repère courant xyz.
        La face du bas, dans le plan z=0, est tracée que si nface == 5 ou 6
        La face du haut, dans le plan z = dz, est tracée que si nface == -5 ou 6
        Ces faces sont centrées en x et y et ont pour coté dx et dy.
        Les trois valeurs dx, dy et dz doivent être positives pour que les
        orientations des faces soient correctes.
---------------------------------------------------------------------------------*/
static void mllBoite(MLLquadricObj *qobj, GLdouble dx, GLdouble dy, GLdouble dz, int nface)
{
    GLfloat x , y, z, zn, nn;


    x = (GLfloat) (dx/2.);
    y = (GLfloat) (dy/2.);
    z = (GLfloat) dz;
    zn = (GLfloat) (dz/2.);

    /* flag direction normale (- si interieur) */
    if (qobj->Orientation == GLU_INSIDE) nn = -1.0;
    else nn = 1.0;

    /* TRACE EN MODE SURFACE */
    if (qobj->DrawStyle == GLU_FILL)
    {
        if (qobj->TextureFlag) {
            glBegin( GL_QUADS );
            glNormal3f( 0., nn, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f( x, y, z);
            glTexCoord2f(1., 0.);
            glVertex3f( x, y, 0.);
            glTexCoord2f(1., 1.);
            glVertex3f(-x, y, 0.);
            glTexCoord2f(0., 1.);
            glVertex3f(-x, y, z);

            glNormal3f( -nn, 0., 0.);
            glTexCoord2f(0., 0.);
            glVertex3f(-x, y, z);
            glTexCoord2f(0., 1.);
            glVertex3f(-x, y, 0.);
            glTexCoord2f(1., 1.);
            glVertex3f(-x, -y, 0.);
            glTexCoord2f(1., 0.);
            glVertex3f(-x, -y, z);

            glNormal3f( 0., -nn, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f(-x, -y, z);
            glTexCoord2f(0., 1.);
            glVertex3f(-x, -y, 0.);
            glTexCoord2f(1., 1.);
            glVertex3f( x, -y, 0.);
            glTexCoord2f(1., 0.);
            glVertex3f( x, -y, z);

            glNormal3f( nn, 0., 0.);
            glTexCoord2f(0., 0.);
            glVertex3f( x, -y, z);
            glTexCoord2f(0., 1.);
            glVertex3f( x, -y, 0.);
            glTexCoord2f(1., 1.);
            glVertex3f( x, y, 0.);
            glTexCoord2f(1., 0.);
            glVertex3f( x, y, z);

            if (nface == 5 || nface == 6) {
                glNormal3f( 0., 0., -nn);
                glTexCoord2f(0., 0.);
                glVertex3f( x, y, 0.);
                glTexCoord2f(0., 1.);
                glVertex3f( x, -y, 0.);
                glTexCoord2f(1., 1.);
                glVertex3f(-x,-y, 0.);
                glTexCoord2f(1., 0.);
                glVertex3f(-x, y, 0.);
            }

            if (nface == -5 || nface == 6) {
                glNormal3f( 0., 0., nn);
                glTexCoord2f(0., 0.);
                glVertex3f( x, y, z);
                glTexCoord2f(0., 1.);
                glVertex3f(-x, y, z);
                glTexCoord2f(1., 1.);
                glVertex3f(-x,-y, z);
                glTexCoord2f(1., 0.);
                glVertex3f( x,-y, z);
            }
            glEnd();
        }
        else
        {
            glBegin( GL_QUADS );
            glNormal3f( 0., nn, 0.);
            glVertex3f( x, y, z);
            glVertex3f( x, y, 0.);
            glVertex3f(-x, y, 0.);
            glVertex3f(-x, y, z);

            glNormal3f( -nn, 0., 0.);
            glVertex3f(-x, y, z);
            glVertex3f(-x, y, 0.);
            glVertex3f(-x, -y, 0.);
            glVertex3f(-x, -y, z);

            glNormal3f( 0., -nn, 0.);
            glVertex3f(-x, -y, z);
            glVertex3f(-x, -y, 0.);
            glVertex3f( x, -y, 0.);
            glVertex3f( x, -y, z);

            glNormal3f( nn, 0., 0.);
            glVertex3f( x, -y, z);
            glVertex3f( x, -y, 0.);
            glVertex3f( x, y, 0.);
            glVertex3f( x, y, z);

            if (nface == 5 || nface == 6) {
                glNormal3f( 0., 0., -nn);
                glVertex3f( x, y, 0.);
                glVertex3f( x, -y, 0.);
                glVertex3f(-x,-y, 0.);
                glVertex3f(-x, y, 0.);
            }

            if (nface == -5 || nface == 6) {
                glNormal3f( 0., 0., nn);
                glVertex3f( x, y, z);
                glVertex3f(-x, y, z);
                glVertex3f(-x,-y, z);
                glVertex3f( x,-y, z);
            }
            glEnd();
        }
    }
    /* TRACE EN MODE GLU_POINT */
    else if (qobj->DrawStyle == GLU_POINT)
    {
        glBegin( GL_POINTS );
        glVertex3f( x, y, 0.);
        glVertex3f(-x, y, 0.);
        glVertex3f(-x,-y, 0.);
        glVertex3f( x,-y, 0.);

        glVertex3f( x, y, z);
        glVertex3f(-x, y, z);
        glVertex3f(-x,-y, z);
        glVertex3f( x,-y, z);
        glEnd();
    }
    /* TRACE EN MODE LIGNE ou SILHOUETTE */
    else if (qobj->DrawStyle==GLU_LINE || qobj->DrawStyle==GLU_SILHOUETTE)
    {
        glBegin( GL_LINE_LOOP );
        glVertex3f( x, y, 0.);
        glVertex3f(-x, y, 0.);
        glVertex3f(-x,-y, 0.);
        glVertex3f( x,-y, 0.);
        glEnd();
        glBegin( GL_LINE_LOOP );
        glVertex3f( x, y, z);
        glVertex3f(-x, y, z);
        glVertex3f(-x,-y, z);
        glVertex3f( x,-y, z);
        glEnd();
        /* Trace des lignes de cote */
        glBegin( GL_LINES );
        glVertex3f( x, y, 0.);
        glVertex3f( x, y, z);

        glVertex3f(-x, y, 0.);
        glVertex3f(-x, y, z);

        glVertex3f(-x,-y, 0.);
        glVertex3f(-x,-y, z);

        glVertex3f( x,-y, 0.);
        glVertex3f( x,-y, z);
        glEnd();
    }
}

/*---------------------------------------------------------------------------------
    Borne de cotés parallèles aux plans du repère courant
        delx (>0) épaisseur de la borne
        rayy (>0) rayon de la borne qui a pour largeur 2*rayy
        profz (>0) hauteur de la borne = profz+ry
        Le fond de la borne se situe dans le plan z = -profz.
        L'arrondi de la borne qui culmine dansle plan z=rayy comporte nface facettes
        Le fond de la borne est fermé par une face si flagfond != 0.
---------------------------------------------------------------------------------*/
static void mllBorne(MLLquadricObj *qobj, GLdouble delx, GLdouble rayy,
              GLdouble profz, int nface, int flagfond)
{
    int i ;
    GLfloat x , y, ry, z, hz, nn, ca, sa;
    double angle;

    x = (GLfloat) (delx/2.);
    ry = (GLfloat) rayy ;
    hz = (GLfloat) profz ;

    /* flag direction normale (- si interieur) */
    if (qobj->Orientation == GLU_INSIDE) nn = -1.0;
    else nn = 1.0;

    /* TRACE EN MODE SURFACE */
    if (qobj->DrawStyle == GLU_FILL)
    {
        if (qobj->TextureFlag) {
            GLfloat v1 = hz/(hz+ry), v2=1.f-v1; ;
            glBegin(GL_POLYGON);
            glNormal3f( nn, 0., 0.);
            glTexCoord2f(1., 0.);
            glVertex3f( x, ry, -hz);
            glTexCoord2f(1., v1);
            glVertex3f( x, ry, 0.);
            for (i=1 ; i < nface  ; i++)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glTexCoord2f(0.5f*(1.f+ca), v1+v2*sa);
                glVertex3f( x, y, z);
            }
            glTexCoord2f(0., v1);
            glVertex3f(x, -ry, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f(x, -ry, -hz);
            glEnd();
            glBegin(GL_POLYGON);
            glNormal3f(-nn, 0., 0.);
            glTexCoord2f(1., 0.);
            glVertex3f(-x, -ry, -hz);
            glTexCoord2f(1., v1);
            glVertex3f(-x, -ry, 0.);
            for (i=nface-1 ; i > 0  ; i--)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glTexCoord2f(0.5f*(1.f+ca), v1+v2*sa);
                glVertex3f(-x, y, z);
            }
            glTexCoord2f(0., v1);
            glVertex3f(-x, ry, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f(-x, ry, -hz);
            glEnd();
            glBegin(GL_QUAD_STRIP);
            glNormal3f( 0., nn, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f( x, ry, -hz);
            glTexCoord2f(1., 0.);
            glVertex3f(-x, ry, -hz);
            glTexCoord2f(0., v1);
            glVertex3f( x, ry, 0.);
            glTexCoord2f(1., v1);
            glVertex3f(-x, ry, 0.);
            for (i=1 ; i < nface  ; i++)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glNormal3f( 0., ca, sa);
                glTexCoord2f(0., v1+v2*sa);
                glVertex3f( x, y, z);
                glTexCoord2f(1., v1+v2*sa);
                glVertex3f(-x, y, z);
            }
            glNormal3f( 0., -nn, 0.);
            glTexCoord2f(0., v1);
            glVertex3f( x, -ry, 0.);
            glTexCoord2f(1., v1);
            glVertex3f(-x, -ry, 0.);
            glTexCoord2f(0., 0.);
            glVertex3f( x, -ry, -hz);
            glTexCoord2f(1., 0.);
            glVertex3f(-x, -ry, -hz);
            glEnd();
            if (flagfond)
            {
                glBegin(GL_QUADS);
                glNormal3f( 0., 0., -nn);
                glVertex3f( x, ry, -hz);
                glVertex3f( x,-ry, -hz);
                glVertex3f(-x,-ry, -hz);
                glVertex3f(-x, ry, -hz);
                glEnd();
            }
        }
        else
        {
            glBegin(GL_POLYGON);
            glNormal3f( nn, 0., 0.);
            glVertex3f( x, ry, -hz);
            glVertex3f( x, ry, 0.);
            for (i=1 ; i < nface  ; i++)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glVertex3f( x, y, z);
            }
            glVertex3f(x, -ry, 0.);
            glVertex3f(x, -ry, -hz);
            glEnd();
            glBegin(GL_POLYGON);
            glNormal3f(-nn, 0., 0.);
            glVertex3f(-x, -ry, -hz);
            glVertex3f(-x, -ry, 0.);
            for (i=nface-1 ; i > 0  ; i--)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glVertex3f(-x, y, z);
            }
            glVertex3f(-x, ry, 0.);
            glVertex3f(-x, ry, -hz);
            glEnd();
            glBegin(GL_QUAD_STRIP);
            glNormal3f( 0., nn, 0.);
            glVertex3f( x, ry, -hz);
            glVertex3f(-x, ry, -hz);
            glVertex3f( x, ry, 0.);
            glVertex3f(-x, ry, 0.);
            for (i=1 ; i < nface  ; i++)
            {
                angle = i*M_PI/nface;
                ca = (GLfloat) cos(angle) ;
                sa = (GLfloat) sin(angle) ;
                y = ry*ca;
                z = ry*sa;
                glNormal3f( 0., ca, sa);
                glVertex3f( x, y, z);
                glVertex3f(-x, y, z);
            }
            glNormal3f( 0., -nn, 0.);
            glVertex3f( x, -ry, 0.);
            glVertex3f(-x, -ry, 0.);
            glVertex3f( x, -ry, -hz);
            glVertex3f(-x, -ry, -hz);
            glEnd();
            if (flagfond)
            {
                glBegin(GL_QUADS);
                glNormal3f( 0., 0., -nn);
                glVertex3f( x, ry, -hz);
                glVertex3f( x,-ry, -hz);
                glVertex3f(-x,-ry, -hz);
                glVertex3f(-x, ry, -hz);
                glEnd();
            }
        }
    }
    /* TRACE EN MODE GLU_POINT */
    else if (qobj->DrawStyle == GLU_POINT)
    {
        glBegin( GL_POINTS );
        glVertex3f( x, ry, -hz);
        glVertex3f( x, ry, 0.);
        for (i=1 ; i < nface  ; i++)
        {
            angle = i*M_PI/nface;
            ca = (GLfloat) cos(angle) ;
            sa = (GLfloat) sin(angle) ;
            y = ry*ca;
            z = ry*sa;
            glVertex3f( x, y, z);
        }
        glVertex3f(x, -ry, 0.);
        glVertex3f(x, -ry, -hz);

        glVertex3f(-x, -ry, -hz);
        glVertex3f(-x, -ry, 0.);
        for (i=nface-1 ; i > 0  ; i--)
        {
            angle = i*M_PI/nface;
            ca = (GLfloat) cos(angle) ;
            sa = (GLfloat) sin(angle) ;
            y = ry*ca;
            z = ry*sa;
            glVertex3f(-x, y, z);
        }
        glVertex3f(-x, ry, 0.);
        glVertex3f(-x, ry, -hz);
        glEnd();
    }
    /* TRACE EN MODE LIGNE ou SILHOUETTE */
    else if (qobj->DrawStyle==GLU_LINE || qobj->DrawStyle==GLU_SILHOUETTE)
    {
        glBegin( GL_LINE_LOOP );
        glVertex3f(x, ry, -hz);
        glVertex3f(x, ry, 0.);
        for (i=1 ; i < nface  ; i++)
        {
            angle = i*M_PI/nface;
            ca = (GLfloat) cos(angle) ;
            sa = (GLfloat) sin(angle) ;
            y = ry*ca;
            z = ry*sa;
            glVertex3f(x, y, z);
        }
        glVertex3f(x, -ry, 0.);
        glVertex3f(x, -ry, -hz);
        glEnd();

        glBegin( GL_LINE_LOOP );
        glVertex3f(-x, -ry, -hz);
        glVertex3f(-x, -ry, 0.);
        for (i=nface-1 ; i > 0  ; i--)
        {
            angle = i*M_PI/nface;
            ca = (GLfloat) cos(angle) ;
            sa = (GLfloat) sin(angle) ;
            y = ry*ca;
            z = ry*sa;
            glVertex3f(-x, y, z);
        }
        glVertex3f(-x, ry, 0.);
        glVertex3f(-x, ry, -hz);
        glEnd();
        /* Trace des lignes de cote */
        glBegin( GL_LINES);
        glVertex3f( x, ry, -hz);
        glVertex3f(-x, ry, -hz);
        glVertex3f( x, ry, 0.);
        glVertex3f(-x, ry, 0.);
        for (i=1 ; i < nface  ; i++)
        {
            angle = i*M_PI/nface;
            ca = (GLfloat) cos(angle) ;
            sa = (GLfloat) sin(angle) ;
            y = ry*ca;
            z = ry*sa;
            glVertex3f( x, y, z);
            glVertex3f(-x, y, z);
        }
        glVertex3f( x, -ry, 0.);
        glVertex3f(-x, -ry, 0.);
        glVertex3f( x, -ry, -hz);
        glVertex3f(-x, -ry, -hz);
        glEnd();
    }
}

/**********************************************************************************/
/*              Construction du dessin du robot                                   */
/**********************************************************************************/
static GLfloat gris040[4] = {0.4,0.4,0.4,1.} ;
static GLfloat gris070[4] = {0.7,0.7,0.7,1.} ;
static GLfloat rouge[4]={1.,0.,0.,1.};
static GLfloat bleu[4]={0.,0.,1.,1.};
static GLfloat jaune[4]={1.,1.,0.,1.};
static GLfloat col_c3[]={178./256.,62./256.,1./256.,1.};

/*********************************************************************************
    Génère les 8 displays listes consécutives GL_Corps des 8 corps d'un rv80 :
    1 socle + 6 corps de robots + 1 doigt de pince (l'autre est symétrique)
    hfut : distance verticale entre socle et bras horizontal.
    lseg : longueur d'un segment de bras.
    La destruction finale se fait simplement par : glDeleteLists(DL_Corps,8);
*********************************************************************************/
GLuint glNewRv80(double size)
{
#define nb_faces_cyl 24
#define nb_faces_bor 12

    GLuint DL_Corps, DL_C0, DL_C1, DL_C2, DL_C3, DL_C4, DL_C5, DL_C6, DL_C7;
    const double hfut = 1.5*size;
    const double lseg = 1.*size;
    const double d_car_c0 = 1.5;			/* coté base carrée */
    const double h_car_c0 = 0.25*lseg;			/* hauteur carré base */
    const double r_cyl_c0 = 0.25*lseg;			/* rayon cylindre base */
    const double h_cyl_c1 = 0.10*lseg;			/* Hauteur cylindre corps 1 */
    const double r_cyl_c1 = (1.05*r_cyl_c0);	/* Rayon cylindre corps 1 */
    const double e_sup_u2 = 0.05*lseg;			/* Epaisseur support axe 2 */
    const double dl_cha_u2 = (r_cyl_c0/sqrt(2.)-e_sup_u2);	/* demi-largeur chappe support axe 2 */
    const double r_sup_u2 = (r_cyl_c0/sqrt(2.));	/* Rayon support axe 2 */
    const double h_sup_u2 = (1.5*r_sup_u2);		/* hauteur support axe 2 */
    const double h_cyl_c0 = (hfut-h_car_c0-h_sup_u2-h_cyl_c1); /* hauteur cylindre base */
    const double h_sup_u3 = (lseg/2.);			/* hauteur support axe 3 */
    const double h_car_c2 = (lseg-h_sup_u3);		/* hauteur carré corps 2 */
    const double l_car_c2 = (2.*dl_cha_u2-0.02*lseg);/* largeur carré corps 2 */
    const double e_sup_u3 = 0.04*lseg;			/* Epaisseur support axe 3 */
    const double dl_cha_u3 = (l_car_c2/2.-e_sup_u3);	/* demi-largeur chappe support axe 3 */
    const double h_car_c3 = (0.7*lseg);			/* hauteur carré corps 3 */
    const double l_car_c3 = (2.*dl_cha_u3-0.02*lseg);/* largeur carré corps 3 */
    const double r_cyl1_c3 = r_sup_u2/cos(22.5*M_PI/180.) ; /* rayon cylindre 1 corps 3 */
    const double e_cyl1_c3 = 0.1*lseg;			/* epaisseur cylindre 1 corps 3 */
    const double r_cyl2_c3 = (0.8*r_cyl1_c3);	/* rayon cylindre 2 corps 3 */
    const double h_cyl_c4 = 0.04*lseg;			/* hauteur cylindre corps 4 */
    const double r_cyl_c4 = 0.06*lseg;           /* rayon cylindre corps 4 */
    const double r_sup_u5 = 0.04*lseg;			/* rayon support axe 5 */
    const double h_sup_u5 = 0.06*lseg;		/* hauteur support axe 5 */
    const double e_cyl2_c3 = 0.1*lseg;			/* epaisseur cylindre 2 corps 3 */
    const double e_cha_u5 = 0.04*lseg;			/* Epaisseur chape axe 5 */
    const double l_c5 = 2.*sqrt(r_cyl2_c3*r_cyl2_c3-r_sup_u5*r_sup_u5); /* largeur corps 5 */
    const double dl_cha_u5 = l_c5/2.-e_cha_u5; /* demi largeur chape axe 5 */
    const double l_sup_u5 = (2.*dl_cha_u5-0.02*lseg);/*  largeur support axe 5 */
    const double h_c5 = 2.*r_sup_u5 ;			/* hauteur corps 5 */
    const double e_c5 = h_c5 ;					/* epaisseur corps 5 */
    const double r_c6 = r_sup_u5 ;				/* rayon corps 6 */
    const double h_c6 = h_c5 ;					/* hauteur corps 5 */
    const double dx_pince=0.5*l_c5;
    const double dy_pince=0.5*h_c5;
    const double dz_pince=0.5*e_c5;
    const double dy_doigt=dy_pince;
    const double dx_doigt=dy_doigt;
    const double dz_doigt= 0.75*dx_pince;

    GLUquadricObj *quadric; /* Objet Quadric générique pour cylindres, cones, sphères etc... */
    MLLquadricObj *quadmll; /* Objet mllQuadric générique pour boite, borne */
    glEnable(GL_CULL_FACE) ;

    /* Création objet quadric */

    quadric = gluNewQuadric();				/* Nouvel objet quadric générique */
    quadmll = mllNewQuadric();				/* Nouvel objet quadric générique */

    /* Numero des displays listes */
    DL_Corps = glGenLists(8);
    DL_C0 = DL_Corps;
    DL_C1 = DL_C0+1 ;
    DL_C2 = DL_C1+1 ;
    DL_C3 = DL_C2+1 ;
    DL_C4 = DL_C3+1 ;
    DL_C5 = DL_C4+1 ;
    DL_C6 = DL_C5+1 ;
    DL_C7 = DL_C6+1 ;


    /* Construction du socle du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est en bas */
    glNewList(DL_C0, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glPushMatrix();
        glColor3fv(gris040);
        glTranslated(0., 0., -hfut); // L'origine générale est celle de C1 et C2 confondues
        /* Base carrée */
        mllBoite(quadmll, d_car_c0, d_car_c0, h_car_c0, -5);
        /* Cylindre au dessus de la base */
        glTranslated(0., 0., h_car_c0);
        glColor3fv(gris070);
        gluCylinder(quadric, r_cyl_c0, r_cyl_c0, h_cyl_c0, nb_faces_cyl, 2);
        glPopMatrix();
    }
    glEndList();	/* DL_C0 */

    /* Construction du corps 1 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C1, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glColor3fv(jaune);
        glPushMatrix();
        /* on descend de la hauteur du support et du cylindre */
        glTranslated(0., 0., -(h_sup_u2+h_cyl_c1));
        gluCylinder(quadric, r_cyl_c1, r_cyl_c1, h_cyl_c1, nb_faces_cyl, 2); /* cylindre */
        glTranslated(0., 0., h_cyl_c1); /* on remonte  de sa hauteur */
        gluDisk(quadric, 0., r_cyl_c1, nb_faces_cyl, 1); /* on ferme sa partie supérieure */
        glTranslated(dl_cha_u2+e_sup_u2/2., 0., h_sup_u2); /* partie avant de la chappe */
        mllBorne(quadmll, e_sup_u2, r_sup_u2, h_sup_u2, nb_faces_bor, 0) ;
        glTranslated(-2*dl_cha_u2-e_sup_u2, 0., 0.); /* partie arrière de la chappe */
        mllBorne(quadmll, e_sup_u2, r_sup_u2, h_sup_u2, nb_faces_bor, 0) ;
        glPopMatrix();
    }
    glEndList();	/* DL_C1 */

    /* Construction du corps 2 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C2, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glColor3fv(jaune);
        glPushMatrix();
        glRotated(90., 1., 0., 0.); /* borne avec z selon -y robot */
        mllBorne(quadmll, l_car_c2, r_sup_u2, h_car_c2, nb_faces_bor, 1) ;
        glRotated(180., 1., 0., 0.); /* borne avec z selon y robot */
        glTranslated(dl_cha_u3+e_sup_u3/2., 0., h_car_c2+h_sup_u3); /* partie avant de la chappe */
        mllBorne(quadmll, e_sup_u3, r_sup_u2, h_sup_u3, nb_faces_bor, 0) ;
        glTranslated(-2*dl_cha_u3-e_sup_u3, 0., 0.); /* partie arrière de la chappe */
        mllBorne(quadmll, e_sup_u3, r_sup_u2, h_sup_u3, nb_faces_bor, 0) ;
        glPopMatrix();
        glTranslated(0., lseg, 0.); /* translatze origine en C3 */
    }
    glEndList();	/* DL_C2 */

    /* Construction du corps 3 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C3, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glColor3fv(jaune);
        glPushMatrix();
        glRotated(90., 1., 0., 0.); /* borne avec z selon -y robot */
        mllBorne(quadmll, l_car_c3, r_sup_u2, h_car_c3, nb_faces_bor, 0) ;
        glRotated(180., 1., 0., 0.); /* borne avec z selon y robot */
        glTranslated(0., 0., h_car_c3); /* on translate de la hauteur du carré corps c3 */
        glRotated(22.5, 0., 0., 1.); /* pour les facettes */
        glColor3fv(col_c3);
        gluQuadricOrientation(quadric, GLU_INSIDE) ;
        gluDisk(quadric, 0., r_cyl1_c3, 8, 1); /* on ferme sa partie arrière */
        gluQuadricOrientation(quadric, GLU_OUTSIDE) ;
        gluCylinder(quadric, r_cyl1_c3, r_cyl1_c3, e_cyl1_c3, 8, 2);
        glTranslated(0., 0., e_cyl1_c3); /* on translate de l'épaisseur du cylindre1 corps 3 */
        gluDisk(quadric, 0., r_cyl1_c3, 8, 1); /* on ferme sa partie avant */
        gluCylinder(quadric, r_cyl2_c3, r_cyl2_c3, e_cyl2_c3, nb_faces_cyl, 2);
        glTranslated(0., 0., e_cyl2_c3); /* on translate de l'épaisseur du cylindre1 corps 3 */
        gluDisk(quadric, 0., r_cyl2_c3, nb_faces_cyl, 1); /* on ferme sa partie avant */
        glPopMatrix();
        glTranslated(0., lseg, 0.); /* translatze origine en C4 */
    }
    glEndList();	/* DL_C3 */

    /* Construction du corps 4 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C4, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glColor3fv(gris040);
        glPushMatrix();
        glRotated(-90., 1., 0., 0.); /* borne avec z selon y robot */
        mllBorne(quadmll, l_sup_u5, r_sup_u5, h_sup_u5, nb_faces_bor, 1) ;
        glTranslated(0., 0., -h_sup_u5); /* on translate de la hauteur de la borne */
        gluDisk(quadric, 0., r_cyl_c4, 4, 1); /* on ferme la partie avant du cylindre */
        glTranslated(0., 0., -h_cyl_c4); /* on translate de la hauteur l'axe 5 */
        gluCylinder(quadric, r_cyl_c4, r_cyl_c4, h_cyl_c4, 4, 2);
        glPopMatrix();
    }
    glEndList();	/* DL_C4 */

    /* Construction du corps 5 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C5, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glColor3fv(jaune);
        glPushMatrix();
        glRotated(90., 1., 0., 0.); /* borne avec -z selon y robot */
        glTranslated(dl_cha_u5+e_cha_u5/2., 0., 0.);
        mllBorne(quadmll, e_cha_u5, r_sup_u5, h_sup_u5, nb_faces_bor, 0) ;
        glTranslated(-2*dl_cha_u5-e_cha_u5, 0., 0.);
        mllBorne(quadmll, e_cha_u5, r_sup_u5, h_sup_u5, nb_faces_bor, 0) ;
        glTranslated(dl_cha_u5+e_cha_u5/2., 0., -h_sup_u5-e_c5);
        mllBoite(quadmll, l_c5, h_c5, e_c5, 5) ;
        glPopMatrix();
    }
    glEndList();	/* DL_C5 */

    /* Construction du corps 6 du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C6, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
        glPushMatrix();
        /* cylindre axe 6 */
        glColor3fv(gris040);
        glRotated(-90., 1., 0., 0.); /* z selon y robot */
        glTranslated(0., 0., h_sup_u5+e_c5);
        gluCylinder(quadric, r_c6, r_c6, h_c6, nb_faces_cyl, 2);
        glTranslated(0., 0., h_c6);
        gluDisk(quadric, 0., r_c6, nb_faces_cyl, 1); /* on ferme sa partie avant */
        /* paume de la pince */
        glColor3fv(gris070);
        mllBoite(quadmll, dx_pince, dy_pince, dz_pince, 6) ;
        glPopMatrix();
    }
    glEndList();	/* DL_C6 */

    /* Construction de la pince du Robot par une DisplayList */
    /* Z est vers le haut et X vers l'avant, l'origine est sur l'axe */
    glNewList(DL_C7, GL_COMPILE);			/* Définition sans execution */
    { /* crochet pour la lisibilité uniquement */
//        glColor3fv(gris070);
        glPushMatrix();
        glRotated(-90., 1., 0., 0.); /* z selon y robot */
        glTranslated(0., 0., h_sup_u5+e_c5+h_c6+dz_pince);
        mllBoite(quadmll, dx_doigt, dy_doigt, dz_doigt, -5) ;
        glPopMatrix();
    }
    glEndList();	/* DL_C7 */

    gluDeleteQuadric(quadric);
    mllDeleteQuadric(quadmll);
    return DL_Corps;
}

/*********************************************************************************
    Dessine les 8 displays listes consécutives GL_Corps des 8 corps d'un rv80 :
    1 socle + 6 corps de robots + 1 doigt de pince (l'autre est symétrique)
    dans une configuration spécifiée par 7 valeurs Ang[0:7] qui sont les :
    - 6 angles de configuration en degrés
    - le demi-écartement des doigts de la pince
*********************************************************************************/
void glDrawRv80(GLuint DL_Corps, double *Ang)
{
    glCallList(DL_Corps+0); /* Base */
    glRotated(Ang[0], 0.0, 0.0, 1.0);
    glCallList(DL_Corps+1); /* Corps C1 */
    glRotated(Ang[1], 1.0, 0.0, 0.0);
    glCallList(DL_Corps+2); /* Corps C2 */
    glRotated(Ang[2], 1.0, 0.0, 0.0);
    glCallList(DL_Corps+3); /* Corps C3 */
    glRotated(Ang[3], 0.0, 1.0, 0.0);
    glCallList(DL_Corps+4); /* Corps C4 */
    glRotated(Ang[4], 1.0, 0.0, 0.0);
    glCallList(DL_Corps+5); /* Corps C5 */
    glRotated(Ang[5], 0.0, 1.0, 0.0);
    glCallList(DL_Corps+6); /* Corps C6 */
    glTranslated(-fabs(Ang[6]), 0., 0.);
    glColor3fv(rouge);
    glCallList(DL_Corps+7); /* Doigts de la pince */
    glTranslated(2*fabs(Ang[6]), 0., 0.);
    glColor3fv(bleu);
    glCallList(DL_Corps+7); /* Doigts de la pince */
}
