#include "robman.h"
#include <math.h>
#include "float.h"
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)

Robman::Robman()
{
    delete labelHelp;
    labelHelp = new QLabel(
            tr(
                    "<h4>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p(minus) pour projection orthogonale</li>"
                    "<li>P(majus) pour perspective</li>"
                    "<li>d - D         : +/- droite</li>"
                    "<li>h - H         : +/- haut</li>"
                    "<li>S mvt soleil avec flèches</li>"
                    "<li>C mvt camera avec flèches</li>"
                    "</ul>"
                    "<h4>Mouvements Camera/Soleil :</h4>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h4>Mouvements robot :</h4>"
                    "<ul><li>   v,V  : Modification pas d'inversion</li>"
                    "<li>0,o,O: Retour origine</li></ul>"
                    "<h5>Mouvements cartésiens : </h5>"
                    "<ul><li>    x, X : +/- Translation en x</li>"
                    "<li>    y, Y : +/- Translation en y</li>"
                    "<li>    z, Z : +/- Translation en z</li>"
                    "<li>    l, L : +/- Rotation en lacet(z)</li>"
                    "<li>    t, T : +/- Rotation en tangage(x)</li>"
                    "<li>    r, R : +/- Rotation en roulis (y)</li></ul>"
                    "<h5>Mouvements articulaires : </h5>"
                    "<ul><li>    1,2..,7 : choix de l'axe</li>"
                    "<li>    +, -    : rotation +/-</li>"
                    "</ul>"
                    ), 0);
    labelHelp->setWindowTitle("Aide Visu Robman");

    font.setPointSize(8); // Pour les textes 2D.

    // Initialisation modele robot rv80
    rv80 = newDataRv80() ;
    for (int i=0;i<3;i++) ltr[i]=0.;

    Ang[0] = 0.;
    Ang[1] = 60.;
    Ang[2] = -120.;
    Ang[3] = 0.;
    Ang[4] = 60.;
    Ang[5] = 0.;
    Ang[6] = 0.04;
    AngleSel = 0 ;

    dqlim = 3. ; /* variation angulaire limite par pas d'inversion */
    okinv = 0 ; /* Nombre d'itérations pour inverser ou < 0 si echec */

//    solOrCam = false;
}
#include "mllglutil.h"
#include "glrv80.h"
void Robman::create()
{
    AziSol = 90.;
    SitSol = 60.;
    DistSol = 150.;
    majsol(PosiSol, DistSol, AziSol*DEG2RAD, SitSol*DEG2RAD);
    DL_Corps = glNewRv80(1.);

    glClearColor(0.85, 0.95, 0.95, 1.);

    // Nombre de paramètres à transmettre par setConfig
    nbpars = 7; //(6 + pince)
}

void Robman::AffichStr(int x, int y, char *string )
{
    wGL->renderText (x, wGL->height1-y, string, font) ;
}


void Robman::redraw(void)
{
    int i;
    char texte[256];
    int N = rv80->nq ;
    double qi[6];
    Por Dom;
    /*-------------------------------
      Positionement des eclairages
     -------------------------------*/
    mlSetLight(GL_LIGHT0, PosiSol);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE); /* Pas de faces internes visibles */

    /*----------------------
      On dessine le Robot
    ----------------------*/
    glDrawRv80(DL_Corps, Ang);

    /*---------------------------
      Impression des positions
    ---------------------------*/
    glDisable(GL_DEPTH_TEST); /* Désactive lignes cachées */
    /*-------------------
       On se met en 2D
     -------------------*/
    glViewport(0, 0, wGL->width1, wGL->height1);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0.0, wGL->width1, 0.0, wGL->height1, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    /*--------------------
    Plus d'éclairage
     --------------------*/
    glDisable(GL_LIGHTING);
    glColor3d(0., 0.,0.); // On ecrit en noir
    /*--------------
     On calcule la situation et on l'imprime
    --------------*/
    scavec(qi, DEG2RAD, Ang, N); /* Configuration courante */
    qi2Dom(rv80, qi, Dom); /* Situation courante */
    mrot2ltr (ltr, Dom+3);

    i = sprintf(texte,"X=%+4.2lf     Y=%+4.2lf     Z=%+4.2lf", Dom[0], Dom[1], Dom[2]);
    sprintf(texte+i,"     Cap=%+5.0lf   Ass=%+5.0lf   Rol=%+5.0lf",
            ltr[0]*180/M_PI, ltr[1]*180/M_PI, ltr[2]*180/M_PI);
    AffichStr (10, wGL->height1-15, texte) ;

    sprintf(texte,"AngleSel=%d  dQl=%4.2lf    OkInv=%d", AngleSel+1, dqlim, okinv);
    AffichStr (10, wGL->height1-30, texte) ;


    sprintf(texte,"Axe1=%+5.0lf  Axe2=%+5.0lf  Axe3=%+5.0lf"
            "  Axe4=%+5.0lf  Axe5=%+5.0lf  Axe6=%+5.0lf  Axe7=%+5.3lf"
            , Ang[0],Ang[1],Ang[2],Ang[3],Ang[4],Ang[5],Ang[6]);
    AffichStr (10, wGL->height1-45, texte) ;

    i = sprintf(texte,"AziCam=%+4.0lf  SitCam=%+4.0lf  DistCam=%+4.0lf", wGL->AziCam, wGL->SitCam, wGL->DistCam);
    sprintf(texte+i,"  AziSol=%+4.0lf  SitSol=%+4.0lf  DistSol=%+4.0lf", AziSol, SitSol, DistSol);
    AffichStr (10, 15+3, texte) ;

    sprintf(texte,"Clic droit pour menu contextuel (Aide)");
    AffichStr (10, 3, texte) ;
    return;
}

void Robman::setConfig(double *data)
{
    int i;
    for (i = 0; i < 7; i++) Ang[i] = data[i];
    wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool Robman::keyConfig(int key, bool majuscule)
{
    bool ok = false;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;
    double DeltaA = 2.5, DeltaL = 0.05;

    switch (key)
    {
    case Qt::Key_0 : /* au centre */
    case Qt::Key_O : /* au centre */
        Ang[0] = 0.;
        Ang[1] = 60.;
        Ang[2] = -120.;
        Ang[3] = 0.;
        Ang[4] = 60.;
        Ang[5] = 0.;
        Ang[6] = 0.04;
        ok = TRUE;
        break;
        //////////////////////////////////////////////////////////////////////////////
    case Qt::Key_Plus :
        if (AngleSel==6) Ang[AngleSel] += DeltaA/1000. ;
        else Ang[AngleSel] += DeltaA ;
        ok = TRUE;
        break ;

    case Qt::Key_Minus :
        if (AngleSel==6) Ang[AngleSel] -= DeltaA/1000. ;
        else Ang[AngleSel] -= DeltaA ;
        ok = TRUE;
        break ;
    case Qt::Key_1:
    case Qt::Key_2:
    case Qt::Key_3:
    case Qt::Key_4:
    case Qt::Key_5:
    case Qt::Key_6:
    case Qt::Key_7:
        AngleSel = key - Qt::Key_1;
        ok = true;
        break ;
    case Qt::Key_V:
        dqlim -= smaj*0.05 ;
        if (dqlim < 0.05) dqlim = 0.05;
        if (dqlim > 10) dqlim = 10.;
        ok = true;
        break ;
    case Qt::Key_L:
    case Qt::Key_T:
    case Qt::Key_R:
    case Qt::Key_X:
    case Qt::Key_Y:
    case Qt::Key_Z:
        {
            int N = rv80->nq ;
            double qi[6];
            double dql[6], mat[9];
            Por Dom;

            scavec(qi,M_PI/180., Ang, N); /* Configuration courante */
            qi2Dom(rv80, qi, Dom); /* Situation courante */

            /* Décode la demande */
            switch (key)
            {
            case Qt::Key_L: m3m3(Dom+3, Dom+3, mrotz(mat, -smaj*DeltaA*DEG2RAD));break;
            case Qt::Key_T: m3m3(Dom+3, Dom+3, mrotx(mat, -smaj*DeltaA*DEG2RAD));break;
            case Qt::Key_R: m3m3(Dom+3, Dom+3, mroty(mat, -smaj*DeltaA*DEG2RAD));break;
            case Qt::Key_X: Dom[0] -= smaj*DeltaL; break;
            case Qt::Key_Y: Dom[1] -= smaj*DeltaL; break;
            case Qt::Key_Z: Dom[2] -= smaj*DeltaL; break;
            }
            /* Geoinv */
            setvec(dql, dqlim, N) ;
            okinv = Dom2qi(rv80, Dom, qi, 1000, 0.0005, 0.0005, 0.2, 0.2, dql);
            if (okinv >= 0) scavec(Ang, RAD2DEG, qi, N); /* Conversion en degrés pour OPenGL */
            ok = true;
        }
        break;
    }
    if (ok) wGL->updateGL();
    return ok;
}

//#include "mllglutil.h"
