#include "rubik.h"
#include "glrubicub.h"
#include <stdlib.h>
#include <QFont>

// Création des donnees d'un rubicube
Rubicub *glNewRubicub(double size)
{
    int *avant, *retro, *haut, *bas, *droite, *gauche;
    GLuint favant, fretro, fhaut,fbas,fgauch,fdroit; /* faces visibles */
    GLuint kavant, kretro, khaut,kbas,kgauch,kdroit; /* faces internes */
    int i,j,k,m,n, ia,ir, ih,ib, id,ig;
    double x,y,z;
    double D = 0.5*size;
    double d = 0.44*size;
    double C = (2.*D);
    Rubicub *r = (Rubicub *) malloc(sizeof(Rubicub));
    if (r==NULL) return r;
    r->dl_cubes = glGenLists(27+12);
    GLuint dl_faces = r->dl_cubes + 27;

    // Liste des numéros initiaux des 9 cubes par face
    double Coins[][3]= {{-D, D, D},  /* 0 */
                        { D, D, D},  /* 1 */
                        { D,-D, D},  /* 2 */
                        {-D,-D, D},  /* 3 */
                        {-D, D,-D},  /* 4 */
                        { D, D,-D},  /* 5 */
                        { D,-D,-D},  /* 6 */
                        {-D,-D,-D}}; /* 7 */
    double coins[8][3];
    double noir[3]    = {0.0, 0.0, 0.0};
    double fonce[3]    = {0.3, 0.3, 0.3};
    double jaune[3]   = {1.0, 1.0, 0.0};
    double vert[3]    = {0.0, 0.55, 0.0};
    double blanc[3]   = {1.0, 1.0, 1.0};
    double bleu[3]    = {0.0, 0.0, 1.0};
    double rouge[3] = {0.784, 0.0, 0.0};
    double orange[3]  = {1.0, 0.3, 0.0};

    for(i=0;i<8;i++) for (j=0;j<3;j++) coins[i][j] = 0.9*Coins[i][j];

    /*
     * Construction des faces
     * Le Z est vers l'avant (centre bleu en nominal)
     * le X vers la droite (centre orange en nominal)
     * le Y vers le haut (centre blanc en nominal)
     */
    kavant = dl_faces;
    glNewList(kavant, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[0]);
            glVertex3dv(Coins[3]);
            glVertex3dv(Coins[2]);
            glVertex3dv(Coins[1]);
        glEnd();
    glEndList();

    favant = dl_faces+1;
    glNewList(favant, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d(-d, d, D);
            glVertex3dv(Coins[0]);
            glVertex3d(-d,-d, D);
            glVertex3dv(Coins[3]);
            glVertex3d(d,-d, D);
            glVertex3dv(Coins[2]);
            glVertex3d(d, d, D);
            glVertex3dv(Coins[1]);
            glVertex3d(-d, d, D);
            glVertex3dv(Coins[0]);
        glEnd();
        glColor3dv(bleu);
        glBegin(GL_QUADS);
            glVertex3d(-d, d, D);
            glVertex3d(-d,-d, D);
            glVertex3d(d,-d, D);
            glVertex3d(d, d, D);
        glEnd();
    glEndList();

    kretro = dl_faces+2;
    glNewList(kretro, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[4]);
            glVertex3dv(Coins[5]);
            glVertex3dv(Coins[6]);
            glVertex3dv(Coins[7]);
        glEnd();
    glEndList();

    fretro = dl_faces+3;
    glNewList(fretro, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d(-d, d,-D);
            glVertex3dv(Coins[4]);
            glVertex3d( d, d,-D);
            glVertex3dv(Coins[5]);
            glVertex3d( d,-d,-D);
            glVertex3dv(Coins[6]);
            glVertex3d(-d,-d,-D);
            glVertex3dv(Coins[7]);
            glVertex3d(-d, d,-D);
            glVertex3dv(Coins[4]);
        glEnd();
        glColor3dv(vert);
        glBegin(GL_QUADS);
            glVertex3d(-d, d,-D);
            glVertex3d( d, d,-D);
            glVertex3d( d,-d,-D);
            glVertex3d(-d,-d,-D);
        glEnd();
    glEndList();

    khaut = dl_faces+4;
    glNewList(khaut, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[0]);
            glVertex3dv(Coins[1]);
            glVertex3dv(Coins[5]);
            glVertex3dv(Coins[4]);
        glEnd();
    glEndList();

    fhaut = dl_faces+5;
    glNewList(fhaut, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d(-d, D, d);
            glVertex3dv(Coins[0]);
            glVertex3d( d, D, d);
            glVertex3dv(Coins[1]);
            glVertex3d( d, D,-d);
            glVertex3dv(Coins[5]);
            glVertex3d(-d, D,-d);
            glVertex3dv(Coins[4]);
            glVertex3d(-d, D, d);
            glVertex3dv(Coins[0]);
        glEnd();
        glColor3dv(blanc);
        glBegin(GL_QUADS);
            glVertex3d(-d, D, d);
            glVertex3d( d, D, d);
            glVertex3d( d, D,-d);
            glVertex3d(-d, D,-d);
        glEnd();
    glEndList();

    kbas = dl_faces+6;
    glNewList(kbas, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[2]);
            glVertex3dv(Coins[3]);
            glVertex3dv(Coins[7]);
            glVertex3dv(Coins[6]);
        glEnd();
    glEndList();

    fbas = dl_faces+7;
    glNewList(fbas, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d( d,-D, d);
            glVertex3dv(Coins[2]);
            glVertex3d(-d,-D, d);
            glVertex3dv(Coins[3]);
            glVertex3d(-d,-D,-d);
            glVertex3dv(Coins[7]);
            glVertex3d( d,-D,-d);
            glVertex3dv(Coins[6]);
            glVertex3d( d,-D, d);
            glVertex3dv(Coins[2]);
        glEnd();
        glColor3dv(jaune);
        glBegin(GL_QUADS);
            glVertex3d( d,-D, d);
            glVertex3d(-d,-D, d);
            glVertex3d(-d,-D,-d);
            glVertex3d( d,-D,-d);
        glEnd();
    glEndList();

    kgauch = dl_faces+8;
    glNewList(kgauch, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[0]);
            glVertex3dv(Coins[4]);
            glVertex3dv(Coins[7]);
            glVertex3dv(Coins[3]);
        glEnd();
    glEndList();

    fgauch = dl_faces+9;
    glNewList(fgauch, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d(-D, d, d);
            glVertex3dv(Coins[0]);
            glVertex3d(-D, d,-d);
            glVertex3dv(Coins[4]);
            glVertex3d(-D,-d,-d);
            glVertex3dv(Coins[7]);
            glVertex3d(-D,-d, d);
            glVertex3dv(Coins[3]);
            glVertex3d(-D, d, d);
            glVertex3dv(Coins[0]);
        glEnd();
        glColor3dv(rouge);
        glBegin(GL_QUADS);
            glVertex3d(-D, d, d);
            glVertex3d(-D, d,-d);
            glVertex3d(-D,-d,-d);
            glVertex3d(-D,-d, d);
        glEnd();
    glEndList();

    kdroit = dl_faces+10;
    glNewList(kdroit, GL_COMPILE);
        glColor3dv(fonce);
        glBegin(GL_QUADS);
            glVertex3dv(Coins[1]);
            glVertex3dv(Coins[2]);
            glVertex3dv(Coins[6]);
            glVertex3dv(Coins[5]);
        glEnd();
    glEndList();

    fdroit = dl_faces+11;
    glNewList(fdroit, GL_COMPILE);
        glColor3dv(noir);
        glBegin(GL_QUAD_STRIP);
            glVertex3d( D, d, d);
            glVertex3dv(Coins[1]);
            glVertex3d( D,-d, d);
            glVertex3dv(Coins[2]);
            glVertex3d( D,-d,-d);
            glVertex3dv(Coins[6]);
            glVertex3d( D, d,-d);
            glVertex3dv(Coins[5]);
            glVertex3d( D, d, d);
            glVertex3dv(Coins[1]);
        glEnd();
        glColor3dv(orange);
            glBegin(GL_POLYGON);
            glVertex3d( D, d, d);
            glVertex3d( D,-d, d);
            glVertex3d( D,-d,-d);
            glVertex3d( D, d,-d);
        glEnd();
    glEndList();

    /*   LES 27 PETITS CUBES numérotés n = 0 à 26,
    tranche par tranche 0 à 8, puis 9 à 17 et 18 à 26  */
    avant = r->facerot[0];
    retro = r->facerot[1];
    haut = r->facerot[2];
    bas = r->facerot[3];
    droite = r->facerot[4];
    gauche = r->facerot[5];

    n = 0;
    ia = ir = ib = ih = id = ig = 0;
    for (k=0, z=-C; k<3; k++, z +=C)
        for (j=0, y=-C; j<3; j++, y +=C)
            for (i=0, x=-C; i<3; i++, x +=C)
    {
        /* Matrice 4x4 : Orientation identité et position des 27 centres */
        for (m=1;m<15;m++) r->m44_cubes[n][m]=0.;
        for (m=0;m<16;m+=5) r->m44_cubes[n][m]=1.;
        r->m44_cubes[n][12]=x;
        r->m44_cubes[n][13]=y;
        r->m44_cubes[n][14]=z;

        /* dimension et couleurs des 6 faces de chaque petit cube */
        glNewList(r->dl_cubes+n, GL_COMPILE);
            if(k>1) glCallList(favant);
            else glCallList(kavant);

            if(k<1) glCallList(fretro);
            else glCallList(kretro);

            if(j>1) glCallList(fhaut);
            else glCallList(khaut);

            if(j<1) glCallList(fbas);
            else glCallList(kbas);

            if(i>1) glCallList(fdroit);
            else glCallList(kdroit);


            if(i<1) glCallList(fgauch);
            else glCallList(kgauch);
        glEndList();

        /* Constitution des listes de petit cube par grande face */
        if (i>1) droite[id++] = n;
        else if (i < 1) gauche[ig++] = n;
        if (j>1) haut[ih++] = n;
        else if (j < 1) bas[ib++] = n;
        if (k>1) avant[ia++] = n;
        else if (k < 1) retro[ir++] = n;

        /* Numéro du petit cube occupant la position initiale */
        r->index[n] = n; n++;
    }
#if 0
    /* Pour vérification : Apres toute rotation on doit avoir les égalites suivantes */
    for (n=0;n<27;n++)
    {

        i = (int)(r->m44_cubes[n][12]/C + 1.+3*(r->m44_cubes[n][13]/C + 1.)+9*(r->m44_cubes[n][14]/C + 1.) +.5);
        if (n != r->index[i])
        {
            printf("\nErreur : Noeud %d se trouve en %d, attendu en %d",n,i,index[i]);
            exit(0);
        }

    }
#endif
    return r;
}

// Libération des donnees du rubicube
void glDelRubicub(Rubicub *r)
{
    if(r)
    {
        glDeleteLists(r->dl_cubes, 27+12);
        free(r);
        r = NULL;
    }
}

void glDrawRubicub(Rubik *R)
{
    if(R->r==NULL) return;

    void permutations(Rubik *R);
    permutations(R);

    int n;
    glFrontFace (GL_CCW) ;   /* Sens traditionnel */
    glCullFace (GL_BACK) ;    /* faces arrières non tracées */
    glEnable (GL_CULL_FACE) ; /* autorise élimination des faces */
    glPolygonMode (GL_FRONT, GL_FILL);/* Peint les faces avant  */
    for(n=0; n<27; n++)
    {
        glPushMatrix() ;
        glMultMatrixd (R->r->m44_cubes[n]);
        glCallList(R->r->dl_cubes+n);
        glPopMatrix() ;
    }
}

/* Calcul ds permutations des axes de rotations Haut, Droite, Bas,...
 * pour que cela corresponde à la vision de l'usager
 */
void permutations(Rubik *R)
{
    double m44[16]; glGetDoublev(GL_MODELVIEW_MATRIX, m44);
/*
    QFont font;
    font.setPointSize(8); // Pour les textes 2D.
    char texte[256];
    sprintf(texte,"%10.3lf %10.3lf %10.2lf", m44[0], m44[1], m44[2]);
    R->wGL->renderText (10, 30, texte, font) ;
    sprintf(texte,"%10.3lf %10.3lf %10.2lf", m44[4], m44[5], m44[6]);
    R->wGL->renderText (10, 50, texte, font) ;
    sprintf(texte,"%10.3lf %10.3lf %10.2lf", m44[8], m44[9], m44[10]);
    R->wGL->renderText (10, 70, texte, font) ;
*/
    double maxA = fabs(m44[2]);
    char av = m44[2] >= 0. ? 'X' : 'x'; // l'avant est en direction de Z
    char nAv = m44[2] >= 0. ? 0 : 1; // l'avant est en direction de Z
    if(fabs(m44[6])> maxA)
    {
        maxA = fabs(m44[6]); av = m44[6] >= 0. ? 'Y' : 'y';
        nAv = m44[6] >= 0. ? 2 : 3;
    }
    if(fabs(m44[10])> maxA)
    {
        maxA = fabs(m44[10]);
        av = m44[10] >= 0. ? 'Z' : 'z';
        nAv = m44[10] >= 0. ? 4 : 5;
    }
    double maxH = fabs(m44[1]);
    char ha = m44[1] >= 0. ? 'X' : 'x'; // Le haut est en direction de +Y
    int nHa = m44[1] >= 0. ? 0 : 1; // Le haut est en direction de +Y
    if(fabs(m44[5])> maxH)
    {
        maxH = fabs(m44[5]); ha = m44[5] >= 0. ? 'Y' : 'y';
        nHa = m44[5] >= 0. ? 2 : 3;
    }
    if(fabs(m44[9])> maxH)
    {
        maxH = fabs(m44[9]); ha = m44[9] >= 0. ? 'Z' : 'z';
        nHa = m44[9] >= 0. ? 4 : 5;

    }

    int nh = nHa/2; // num axe haut 0, 1, 2
    int sh = 1 - nHa%2; // sens axe haut : 1 direct, 0 inverse
    int na = nAv/2; // num axe avant 0, 1, 2
    int sa = 1 - nAv%2; // sens axe avant : 1 direct, 0 inverse
    int suiv = (na == ((nh+1)%3));
    int nd = suiv ? (na+1)%3 : (nh+1)%3;
    int sd = (sh == suiv) ? sa : 1-sa;
    int nDr = 2*nd + (1 - sd);
/*
    char coco[][8] = {"orange", "rouge", "blanc", "jaune", "bleu", "vert"};
    sprintf(texte,"Haut %s, Avant %s, Droite %s", coco[nHa], coco[nAv], coco[nDr]);
    R->wGL->renderText (10, 100, texte, font) ;
*/
    R->nAv = nAv;
    R->nHa = nHa;
    R->nDr = nDr;
}
