#include "rubik.h"

Rubik::Rubik()
{
    delete labelHelp;
    labelHelp = new QLabel(
            tr(
                    "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p(minus) pour projection orthogonale</li>"
                    "<li>P(majus) pour perspective</li></ul>"
                    "<h5>Mouvements observateur :</h5>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h5>Mouvements rubik :</h5>"
                    "<li>a,A : rotation face Avant   sens direct,inverse</li>"
                    "<li>r,R : rotation face aRrière sens direct,inverse</li>"
                    "<li>h,H : rotation face Haut    sens direct,inverse</li>"
                    "<li>b,B : rotation face Bas     sens direct,inverse</li>"
                    "<li>d,D : rotation face Droite  sens direct,inverse</li>"
                    "<li>g,G : rotation face Gauche  sens direct,inverse</li>"
                    "<li>@   : refait le cube par la séquence inverse</li>"
                    "<li>Barre espace : Play/Pause rotation en cours<li></ul>"
                    ), 0);    
    labelHelp->setWindowTitle("Aide Visu Rubik");

    frottra = 0;  // pas de rotation en cours
    timertra = new QTimer(this);
    timertra->setSingleShot(false);
    connect(timertra, SIGNAL(timeout()), this, SLOT(rottra()));
    deltaT = 50; // intervalle rafraichissement en ms
    timertra->setInterval(deltaT);
    enreg = true;
    listEnreg.clear();
}

void Rubik::create()
{
    r = glNewRubicub(1.);
    glClearColor(0.85, 0.95, 0.95, 1.0);
#if 1 // Le rubik est dessine ds les axes openGL avec z horizontal
    wGL->flagZvertical = false;
    wGL->SitCam = wGL->SitCamIni = 30.;
    wGL->AziCam = wGL->AziCamIni = 70.;
#endif
    return ;
}

void Rubik::redraw(void)
{
    if(wGL->flagZvertical) // Au cas où compilé avec #if 0 ci-dessus.
    {
        glRotated( 90.,0.,0.,1.);
        glRotated( 90.,1.,0.,0.);
    }
    glDrawRubicub(this);
}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation.
void Rubik::setConfig(double *data)
{
//    wGL->updateGL();
}

/*-----------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                           */
/* Peut être utilisée ou suchargée.                                            */
/* a,A : rotation face Avant   sens direct,inverse                             */
/* r,R : rotation face aRrière sens direct,inverse                             */
/* h,H : rotation face Haut    sens direct,inverse                             */
/* b,B : rotation face Bas     sens direct,inverse                             */
/* d,D : rotation face Droite  sens direct,inverse                             */
/* g,G : rotation face Gauche  sens direct,inverse                             */
/*-----------------------------------------------------------------------------*/
bool Rubik::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = 1 - 2*maj;
    //    double DeltaL = DistCamIni/100.;
    int ndrot[6] = {5, 6, 3, 4, 1, 2};
    int sdrot[6] = {1, 0, 1, 0, 1, 0};
    int nirot[6] = {6, 5, 4, 3, 2, 1};
    int sirot[6] = {0, 1, 0, 1, 0, 1};
    int noaxe[6] = {0, 0, 1, 1, 2, 2};

    if (!frottra)
    {
        switch (key) // non actif si rotation en cours
        {
#if 0
        case Qt::Key_A : /* avant direct/inverse */
            frottra = 1; iaxerot = 2; sensRot = -smaj; ok = TRUE; break;
        case Qt::Key_R : /* retro direct/inverse */
            frottra = 2; iaxerot = 2; sensRot =  smaj; ok = TRUE; break;
        case Qt::Key_H : /* haut direct/inverse */
            frottra = 3; iaxerot = 1; sensRot = -smaj; ok = TRUE; break;
        case Qt::Key_B : /* bas direct/inverse */
            frottra = 4; iaxerot = 1; sensRot =  smaj; ok = TRUE; break;
        case Qt::Key_D : /* droit direct/inverse */
            frottra = 5; iaxerot = 0; sensRot = -smaj; ok = TRUE; break;
        case Qt::Key_G : /* gauche direct/inverse */
            frottra = 6; iaxerot = 0; sensRot =  smaj; ok = TRUE; break;
#endif
        case Qt::Key_A : /* avant direct/inverse */
            frottra = ndrot[nAv]; iaxerot = noaxe[nAv]; sensRot = sdrot[nAv] ? -smaj : smaj; ok = TRUE; break;
        case Qt::Key_R : /* retro direct/inverse */
            frottra = nirot[nAv]; iaxerot = noaxe[nAv]; ; sensRot =  sdrot[nAv] ? smaj : -smaj; ok = TRUE; break;
        case Qt::Key_H : /* haut direct/inverse */
            frottra = ndrot[nHa]; iaxerot = noaxe[nHa]; sensRot = sdrot[nHa] ? -smaj : smaj; ok = TRUE; break;
        case Qt::Key_B : /* bas direct/inverse */
            frottra = nirot[nHa]; iaxerot = noaxe[nHa]; sensRot =  sdrot[nHa] ? smaj : -smaj; ok = TRUE; break;
        case Qt::Key_D : /* droit direct/inverse */
            frottra = ndrot[nDr]; iaxerot = noaxe[nDr]; sensRot = sdrot[nDr] ? -smaj : smaj; ok = TRUE; break;
        case Qt::Key_G : /* gauche direct/inverse */
            frottra = nirot[nDr]; iaxerot = noaxe[nDr]; sensRot =  sdrot[nDr] ? smaj : -smaj; ok = TRUE; break;
        case Qt::Key_At:
            if (!listEnreg.isEmpty()) frottra = 7; ok = true; break;
        }

        if (frottra)  // Lancement rotation
        {           
            AngRot = 0.; /* Debut rotation */
            if (frottra == 7) // Détricottage
            {
                enreg = false;
                int *fsi = listEnreg.takeLast();
                frottra = fsi[0]; sensRot = fsi[1]; iaxerot = fsi[2];
                delete fsi;
            }
            else
            {
                int *fsi = new int[3];
                // on enregistre le sens inverse
                fsi[0] = frottra; fsi[1] = -sensRot; fsi[2] =  iaxerot;
                listEnreg.append(fsi);
            }
            timertra->start();
        }
    }
    else
    {
        switch (key)
        {
        case Qt::Key_Space :
           if(timertra->isActive()) timertra->stop();
           else timertra->start();
           ok = TRUE;
           break ;
        }

    }
    return ok;
}

#define permut(a,b,c,d) k4 = r->index[a]; r->index[a] = r->index[b]; r->index[b] = r->index[c]; r->index[c] = r->index[d]; r->index[d] = k4

/*----------------------------------------------------------------------------------
 Tâche de fond gérant la rotation d'une tranche
----------------------------------------------------------------------------------*/
void Rubik::rottra(void)
{
    int i, k4, flagfin = 0, *face;
    double  ang, angtot, c, s, *M, bid;

    ang = (3.1416/2.)*deltaT/1000. ; // vitesse rotation = pi/2 en une seconde
    angtot = AngRot + ang ;
    if (angtot >= M_PI/2)
    {
        angtot = M_PI/2;
        ang = angtot - AngRot;
        flagfin = 1;
        timertra->stop();
    }
    AngRot = angtot;
    c = cos(ang) ;
    s = sin(ang); if(sensRot < 0) s = -s;

    face = r->facerot[frottra-1]; /* Face à faire tourner */
    for (i=0;i<9;i++) /* Rotation des 9 cubes de la face concernée */
    {
        M = r->m44_cubes[r->index[face[i]]];
        switch(iaxerot)
        {
        case 0: /* rotation x */
            for(k4 = 0; k4 < 16; k4 += 4)
            { bid = M[k4+1]; M[k4+1] = c*bid-s*M[k4+2]; M[k4+2] = s*bid+c*M[k4+2]; }
            break ;
        case 1: /* rotation y */
            for(k4 = 0; k4 < 16; k4 += 4)
            { bid = M[k4]; M[k4] = c*bid+s*M[k4+2]; M[k4+2] = -s*bid+c*M[k4+2]; }
            break ;
        case 2: /* rotation z */
            for(k4 = 0; k4 < 16; k4 += 4)
            { bid = M[k4]; M[k4] = c*bid-s*M[k4+1]; M[k4+1] = s*bid+c*M[k4+1]; }
            break ;
        }
    }
    wGL->updateGL();

    if (flagfin) /* On met a jour les nouvelles positions occupées par les petits cubes */
    {
        switch (frottra)
        {
        case 1 : // AVANT
            if(sensRot > 0) { permut(18,24,26,20); permut(19,21,25,23);}
            else { permut(20,26,24,18); permut(23,25,21,19);}
            break;
        case 2 :// RETRO
            if(sensRot < 0){ permut(0,2,8,6); permut(1,5,7,3);}
            else { permut(6,8,2,0); permut(3,7,5,1);}
            break;
        case 3 : // HAUT
            if(sensRot > 0){ permut(6,8,26,24); permut(7,17,25,15);}
            else { permut(24,26,8,6); permut(15,25,17,7);}
            break;
        case 4 : // BAS
            if(sensRot < 0) { permut(0,18,20,2); permut(1,9,19,11);}
            else { permut(2,20,18,0); permut(11,19,9,1);}
            break;
        case 5 :  // DROITE
            if(sensRot > 0){ permut(2,20,26,8); permut(11,23,17,5);}
            else { permut(8,26,20,2); permut(5,17,23,11);}
            break;
        case 6 : // GAUCHE
            if(sensRot < 0) { permut(0,6,24,18); permut(9,3,15,21);}
            else { permut(18,24,6,0); permut(21,15,3,9);}
            break;
        }
        if (enreg) frottra = 0;
        else
        {   // Examine si reste face à détricoter
            if(!listEnreg.isEmpty())
            {
                int *fsi = listEnreg.takeLast();
                frottra = fsi[0]; sensRot = fsi[1]; iaxerot = fsi[2];
                delete fsi;
                AngRot = 0.; /* Debut rotation */
                timertra->start();
            }
            else {enreg = true; frottra = 0;}
        }
    }
}
