#include "tetragyre.h"
#include "GL/glu.h"
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)
#include "mllglutil.h"

static void madvec(double* w, double* a, double* v)
{
    double p, q ;
    p  = a[0] * v[0] + a[3] * v[1] + a[6] * v[2];
    q  = a[1] * v[0] + a[4] * v[1] + a[7] * v[2];
    w[2] = a[2] * v[0] + a[5] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
}

static void madmad(double* a,double* b, double* c)
{
    register int i ;
    double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[1] + b[6] * c[2];
    d[1] = b[1] * c[0] + b[4] * c[1] + b[7] * c[2];
    d[2] = b[2] * c[0] + b[5] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[3] * c[4] + b[6] * c[5];
    d[4] = b[1] * c[3] + b[4] * c[4] + b[7] * c[5];
    d[5] = b[2] * c[3] + b[5] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[3] * c[7] + b[6] * c[8];
    d[7] = b[1] * c[6] + b[4] * c[7] + b[7] * c[8];
    a[8] = b[2] * c[6] + b[5] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
}

static void mrotx(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[0] = 1.;
    a[1] = a[2] = a[3] = a[6] = 0.;
    a[4] = a[8]= c;
    a[5] = s; a[7] = -s;
}

static void mroty(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[4] = 1.;
    a[1] = a[3] = a[5] = a[7] = 0.;
    a[0] = a[8]= c;
    a[6] = s; a[2] = -s;
}

static void mrotz(double* a, double g)
{
    double s, c;
    c = cos(g);
    s = sin(g);
    a[8] = 1.;
    a[2] = a[5] = a[6] = a[7] = 0.;
    a[0] = a[4]= c;
    a[1] = s; a[3] = -s;
}

static double *mat44from33(double *m44, double *m33)
{
    m44[0] = m33[0] ; m44[4] = m33[3] ;  m44[8]  = m33[6] ; m44[12] = 0. ;
    m44[1] = m33[1] ; m44[5] = m33[4] ;  m44[9]  = m33[7] ; m44[13] = 0. ;
    m44[2] = m33[2] ; m44[6] = m33[5] ;  m44[10] = m33[8] ; m44[14] = 0. ;
    m44[3] = 0.     ; m44[7] = 0.     ;  m44[11] = 0.     ; m44[15] = 1. ;
    return m44 ;
}


Tetragyre::Tetragyre()
{
    dimConf = 7;
//#define ELOING 300.
//#define SITE 20.0
//#define AZIMUT 60.0
//    DistCam = DistCamIni = ELOING; /* de l'observateur */
//    AziCam = AziCamIni = AZIMUT;
//    SitCam = SitCamIni = SITE; /* Direction de l'observateur */

    sigsel = 1; // axe sélectionné

    delete labelHelp;
    labelHelp = new QLabel(
            tr(
                    "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                    "<ul><li>p(minus) pour projection orthogonale</li>"
                    "<li>P(majus) pour perspective</li></ul>"
                    "<h5>Mouvements observateur :</h5>"
                    "<ul>"
                    "<li>Touches e-E     : +/- éloignement</li>"
                    "<li>Touches Deb-Fin : Droite - Gauche</li>"
                    "<li>Page Haut-Bas   : Haut - Bas</li>"
                    "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                    "<li>Fleches ^ - V   : rotation site</li>"
                    "<li>ESC             : point de vue initial</li></ul>"
                    "<h5>Mouvements maquette :</h5>"
                    "<ul><li>0,o,O: Retour origine</li>"
                    "<li>1,2,3,4  : Sélection axe gyro</li>"
                    "<li> + -     : Rotation axe sel. sens +/-</li></ul>"
                    ), 0);
    labelHelp->setWindowTitle("Aide Tetragyre");

    DeltaA = 2.5;
}




void Tetragyre::create()
{
#define NDIV 40

    int i, j ;
    double zerov[3] = {0., 0., 0.};
    const double Beta = atan(1./sqrt(3.)) ;
    const unsigned char colGrisclair[]={220,220,220};
    const unsigned char colGrisfonce[]={100,100,100};
    const unsigned char colPlatform[]={250,200,50};
    const unsigned char colSupMot[]={250,200,50};
    const unsigned char colCylGyro[]={200,000,000};
    const unsigned char colDisGyro[]={250,050,050};
    const unsigned char colCapot[]={246,162,134,150};
    double size = 0.04;
    double rDisk = 38.*size;

    GLUtesselator *tessplatform ;
    GLUquadricObj *quadplatform, *quadgyro, *quadcolon, *quadcroix ;

    double supcardhaut[8][3] = {
        { 4.*size,  10.*size,   0.},
        {-4.*size,  10.*size,   0.},
        {-4.*size,  10.*size, -15.*size},
        { 4.*size,  10.*size, -15.*size},
        { 4.*size, -10.*size,   0.},
        {-4.*size, -10.*size,   0.},
        {-4.*size, -10.*size, -15.*size},
        { 4.*size, -10.*size, -15.*size}
    };

    double supcardbas[8][3] = {
        { 10.*size,-4.*size,  4.*size},
        { 10.*size, 4.*size,  4.*size},
        { 10.*size,-4.*size,-10.*size},
        { 10.*size, 4.*size,-10.*size},
        {-10.*size,-4.*size,-10.*size},
        {-10.*size, 4.*size,-10.*size},
        {-10.*size,-4.*size,  4.*size},
        {-10.*size, 4.*size,  4.*size},
    };


    double untrou[4][3] = {
        {19.5*size, 7.*size,0.},
        {35.0*size, 7.*size,0.},
        {35.0*size,-7.*size,0.},
        {19.5*size,-7.*size,0.}
    };
    double sup1[5][3]={
        {35.*size, 7.*size,   0.},
        {35.*size, 7.*size, -12.0*size},
        {29.*size, 7.*size, -12.0*size},
        {21.*size, 7.*size,  -4.5*size},
        {21.*size, 7.*size,   0.}
    } ;
    double disque[NDIV][3] ;
    double sup2[5][3], supports[8][5][3], trous[4][4][3];
    double matz[4][9], matbet[9], bid[9], mat[16] ;
    double axeprec[3] = {31.*size, 0., -9.*size};
    double distpg = 24.*size ;
    double axegyr1[3] ;
    double hcentre = 11.*size;

    for(i=1;i<4;i++) sigma[i]=0.;

    // Pourtour du disque
    for (i=0 ; i < NDIV ; i++)
    {
        double a, ca, sa;
        a = (i*2)*M_PI/NDIV ; ca = cos(a) ; sa = sin(a) ;
        disque[i][0] = rDisk*ca ;
        disque[i][1] = rDisk*sa ;
        disque[i][2] = 0. ;
    }

    // support autre coté
    for(i=0 ; i<5; i++)
    {
        sup2[i][0] =  sup1[i][0] ;
        sup2[i][1] = -sup1[i][1] ;
        sup2[i][2] =  sup1[i][2] ;
    }

    // Position centre gyro
    axegyr1[0] = axeprec[0] + distpg*cos(Beta) ;
    axegyr1[1] = axeprec[1] ;
    axegyr1[2] = axeprec[2] - distpg*sin(Beta) + hcentre ;

    // Quelques pré-calculs de coordonnées et matrices
    mroty(matbet,Beta+M_PI/2.) ;
    mrotx(mat,M_PI/2.);
    for (j=0; j<4 ; j++)
    {
        mrotz(matz[j],(j+2)*M_PI/2.);
        madvec(axegyrs[j], matz[j], axegyr1) ;
        for(i=0; i<4; i++)
        {
            madvec(trous[j][i], matz[j], untrou[i]) ;
            madvec(supports[j][i], matz[j], sup1[i]) ;
            madvec(supports[j+4][i], matz[j], sup2[i]) ;
        }
        madvec(supports[j][4], matz[j], sup1[4]) ;
        madvec(supports[j+4][4], matz[j], sup2[4]) ;
        madmad(bid, matz[j], matbet) ;
        madmad(bid, bid, mat) ;
        posEtmat2GLmat44(Matgyrs[j], zerov, bid);
    }

    // Modes sélctionnés par défaut
    glEnable(GL_DEPTH_TEST);	// Zbuffer activé
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL) ;
    glEnable(GL_CULL_FACE);		// Pré-élimine faces arrières
    glFrontFace(GL_CCW);		// CCW par défaut

    // Pour la transparence du capot
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;

    quadplatform = gluNewQuadric() ;
    quadcolon = gluNewQuadric() ;
    quadcroix = gluNewQuadric() ;
    quadgyro = gluNewQuadric() ;
    tessplatform = gluNewTess() ;
#define _GLUfuncptr void (CALLBACK *)()
    gluTessCallback (tessplatform, GLU_TESS_BEGIN, (void (CALLBACK *) ()) glBegin) ;
    gluTessCallback (tessplatform, GLU_TESS_END, glEnd) ;
    gluTessCallback (tessplatform, GLU_TESS_VERTEX, (void (CALLBACK *) ()) glVertex3dv) ;

    // La colonne
    Colonne = glGenLists(1);
    glNewList(Colonne, GL_COMPILE);
    glColor3ubv(colGrisfonce) ;
    glPushMatrix();
    glTranslated(0., 0., -68.*size) ;
    glFrontFace(GL_CW);
    gluDisk(quadcolon, 0., 18.*size, NDIV, 1) ;
    glFrontFace(GL_CCW);
    gluCylinder(quadcolon, 18.*size, 18.*size, 2.*size, NDIV, 1) ; // -66
    glTranslated(0., 0., 2.*size) ;
    gluCylinder(quadcolon, 18.*size, 10.*size, 8.*size, NDIV, 1) ; // -58
    glTranslated(0., 0., 8.*size) ;
    gluCylinder(quadcolon, 10.*size, 10.*size, 12.*size, NDIV, 1) ; // -46
    glTranslated(0., 0., 12.*size) ;
    gluCylinder(quadcolon, 10.*size, 6.*size, 4.*size, NDIV, 1) ; // -42
    glTranslated(0., 0., 4.*size) ;
    gluCylinder(quadcolon, 6.*size, 6.*size, 30.*size, NDIV, 1) ; // -12
    glTranslated(0., 0., 30.*size) ;
    gluCylinder(quadcolon, 6.*size, 4.*size, 2.*size, NDIV, 1) ; // -10
    glTranslated(0., 0., 2.*size) ;
    glPopMatrix();
    glEndList();

    // Le support lacet
    Suplacet = glGenLists(1);
    glNewList(Suplacet, GL_COMPILE);
    glDisable(GL_CULL_FACE);
    glColor3ubv(colGrisclair) ;
    glBegin(GL_QUAD_STRIP);
    for (i=0 ; i<8 ; i++) glVertex3dv(supcardbas[i]);
    glEnd() ;
    glEnable(GL_CULL_FACE);
    glEndList();

    // La Croix
    Croix = glGenLists(1);
    glNewList(Croix, GL_COMPILE);
    glPushMatrix();
    glFrontFace(GL_CW);
    glColor3ubv(colGrisfonce) ;
    glRotated(90.,0.,1.,0.) ;
    glTranslated(0.,0.,-10.*size) ;
    gluCylinder(quadcroix, 2.*size, 2.*size, 20.*size, 6, 1) ;
    glPopMatrix();
    glPushMatrix();
    glRotated(90.,1.,0.,0.) ;
    glTranslated(0.,0.,-10.*size) ;
    gluCylinder(quadcroix, 2.*size, 2.*size, 20.*size, 6, 1) ;
    glFrontFace(GL_CCW);
    glPopMatrix();
    glEndList();

    // La plate-forme
    Platsup = glGenLists(1);
    glNewList(Platsup, GL_COMPILE);
    glPushMatrix();

    glTranslated(0.,0., hcentre) ;
    glDisable(GL_BLEND) ;
    glDisable(GL_CULL_FACE);

    // Les supports cardan
    glColor3ubv(colGrisclair) ;
    glBegin(GL_QUADS);
    for (i=0 ; i<8 ; i++) glVertex3dv(supcardhaut[i]);
    glEnd() ;

    // Le disque et ses 4 trous
    glColor3ubv(colPlatform) ;
    gluTessBeginPolygon(tessplatform, NULL);
    gluTessBeginContour(tessplatform) ;
    for (i=0 ; i < NDIV ; i++)
        gluTessVertex(tessplatform, disque[i], disque[i]) ;
    for (j=0; j<4 ; j++)
    {
        gluTessEndContour(tessplatform) ;
        gluTessBeginContour(tessplatform) ;
        for(i=0; i<4; i++)
            gluTessVertex(tessplatform, trous[j][i], trous[j][i]) ;
    }
    gluTessEndContour(tessplatform) ;
    gluTessEndPolygon(tessplatform);

    // Les 8 supports
    glColor3ubv(colSupMot) ;
    for (j=0; j<8 ; j++)
    {
        glBegin(GL_POLYGON) ;
        for(i=0; i<5; i++)
            glVertex3dv(supports[j][i]) ;
        glEnd() ;
    }

    // Les 4 moteurs
    for (j=0; j<4 ; j++)
    {
        glPushMatrix();
        glMultMatrixd(mat44from33(mat,matz[j]));
        glTranslated(axeprec[0],axeprec[1],axeprec[2]);
        glRotated(90+Beta*180/M_PI,0.,1.,0.);
        glTranslated(0. ,0., -14.*size);
        glColor3ubv(colGrisclair) ;
        gluCylinder(quadplatform, 6.*size, 6.*size, 26.*size, NDIV, 1) ;
        glColor3ubv(colGrisfonce) ;
        gluCylinder(quadplatform, 3.*size, 3.*size, 28.*size, 4, 1) ;
        glPopMatrix();
    }

    // le capot
    glEnable(GL_BLEND) ;
    glColor4ubv(colCapot) ;
    glPushMatrix();
    gluCylinder(quadplatform, rDisk, rDisk, 13.*size, NDIV, 1) ;
    glTranslated(0., 0., 13.*size);
    gluDisk(quadplatform, 0., rDisk, NDIV, 1) ;
    glPopMatrix();
    glPopMatrix() ;
    glEnable(GL_CULL_FACE);
    glEndList();

    // Un gyrodyne
    UnGyro = glGenLists(1);
    glNewList(UnGyro, GL_COMPILE);
    glPushMatrix();
    glTranslated(0., 0., -11.*size);
    glFrontFace(GL_CW);
    glColor3ubv(colGrisfonce) ;
    gluDisk(quadgyro, 0., 1.5*size, NDIV, 1) ;
    glFrontFace(GL_CCW);
    glColor3ubv(colGrisclair) ;
    gluCylinder(quadgyro, 1.5*size, 1.5*size, 6.*size, NDIV, 1) ;
    glTranslated(0., 0., 6.*size);
    glFrontFace(GL_CW);
    glColor3ubv(colDisGyro) ;
    gluDisk(quadgyro, 0., 10.*size, NDIV, 1) ;
    glFrontFace(GL_CCW);
    glColor3ubv(colCylGyro) ;
    gluCylinder(quadgyro, 10.*size, 10.*size, 10.*size, NDIV, 1) ;
    glTranslated(0., 0., 10.*size);
    glColor3ubv(colDisGyro) ;
    gluDisk(quadgyro, 0., 10.*size, NDIV, 1) ;
    glColor3ubv(colGrisclair) ;
    gluCylinder(quadgyro, 1.5*size, 1.5*size, 6.*size, NDIV, 1) ;
    glTranslated(0., 0., 6.*size);
    glColor3ubv(colGrisfonce) ;
    gluDisk(quadgyro, 0., 1.5*size, NDIV, 1) ;
    glPopMatrix();
    glEndList();

    gluDeleteTess(tessplatform) ;
    gluDeleteQuadric(quadplatform) ;
    gluDeleteQuadric(quadgyro) ;
    gluDeleteQuadric(quadcroix) ;
    gluDeleteQuadric(quadcolon) ;

    // Initialise La vue
    glClearColor(0.85, 0.95, 0.95, 1.0);
    glShadeModel(GL_FLAT);
    glEnable(GL_DEPTH_TEST);

    for (int i=0; i< dimConf; i++) sigma[i] = 0.;
    nbpars = dimConf;
    return ;
}

// Exemple de dessin de la scene exemple decrite ci-avant.
void Tetragyre::redraw(void)
{
    GLfloat pos_lumiere1[]={1.,0.,1.,0.}; /* selon bissectrice XZ */
    mlSetLight(GL_LIGHT0, pos_lumiere1);

    glTranslated(0.,0.,0.6);
    // On trace la scène
    glCallList(Colonne);
    glRotated(sigma[4],0.,0.,1.);
    glCallList(Suplacet);
    glRotated(sigma[5],1.,0.,0.);
    glCallList(Croix);
    glRotated(sigma[6],0.,1.,0.);
    glCallList(Platsup);
    for (int j = 0 ; j < 4 ; j++)
    {
        glPushMatrix();
        glTranslated(axegyrs[j][0],axegyrs[j][1],axegyrs[j][2]);
        glMultMatrixd(Matgyrs[j]);
        glRotated(sigma[j], 0., 1., 0.);
        glCallList(UnGyro);
        glPopMatrix();
    }
}

// Réception des "dimConf" paramètres envoyés par le
// gestionnaire de l'avance de la simulation. Ici un exemple basique
// de reception des 6 paramètres X,Y,Z,L,T,R
void Tetragyre::setConfig(double *data)
{
    for (int i = 0; i < dimConf; i++) sigma[i] = data[i];
    wGL->updateGL();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches pour actions diverses                                */
/* Peut être utilisée ou suchargée.                                                 */
/* 1,2,3,4  : selection axe commande par les touches + ou -                         */
/* +,-      : rotatin de l'axe selectionne                                          */
/* x,X      : rotation autour de OX (axe support)                                   */
/* y,Y      : rotation autour de OY (axe support)                                   */
/* z,Z      : rotation autour de OZ (axe colonne)                                   */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
bool Tetragyre::keyConfig(int key, bool majuscule)
{
    bool ok = FALSE;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;

    switch (key)
    {
    case Qt::Key_0 : /* au centre */
    case Qt::Key_O : /* au centre */
        for(int i = 0; i < 7; i++) sigma[i] = 0.;
        ok = TRUE;
        break;
    case Qt::Key_1 :
    case Qt::Key_2 :
    case Qt::Key_3 :
    case Qt::Key_4 :
        sigsel = key - Qt::Key_1;
        ok = TRUE;
        break ;
    case Qt::Key_Minus :
        sigma[sigsel] -= DeltaA ;
        ok = TRUE;
        break ;
    case Qt::Key_Plus :
        sigma[sigsel] += DeltaA ;
        ok = TRUE;
        break ;
    case Qt::Key_X :
        sigma[5] -= DeltaA*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_Y :
        sigma[6] -= DeltaA*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_Z :
        sigma[4] -= DeltaA*smaj ;
        ok = TRUE;
        break ;
    }
    if(ok) wGL->updateGL();
    return ok;
}
