#include <stdio.h>
#include <stdlib.h>
#include "textbmap.h"

static unsigned short read_word(FILE *fp)
{
unsigned char b0, b1;
    b0 = getc(fp);
    b1 = getc(fp);
    return ((b1 << 8) | b0);
}

static unsigned long read_dword(FILE *fp)
{
unsigned char b0, b1, b2, b3;
    b0 = getc(fp);
    b1 = getc(fp);
    b2 = getc(fp);
    b3 = getc(fp);
    return ((((((b3 << 8) | b2) << 8) | b1) << 8) | b0);
}

static long read_long(FILE *fp)
{
unsigned char b0, b1, b2, b3;
    b0 = getc(fp);
    b1 = getc(fp);
    b2 = getc(fp);
    b3 = getc(fp);
    return ((long)(((((b3 << 8) | b2) << 8) | b1) << 8) | b0);
}

static unsigned char *loadBmpPix(const char *filename, long *width, long *height)
{
    FILE *fp;
    unsigned char *bits, *ptr, temp;
    int x, y, length;
    unsigned long offData, uSizeImage, lbid;
    unsigned short nbBitsParPixel, sbid;
    long largeur, hauteur;

    if ((fp = fopen(filename, "rb")) == NULL) return (NULL);
    /* Test présence "BM" */
    sbid = read_word(fp);
    if (sbid != 0x4D42) { fclose(fp); return (NULL); }

    fseek(fp, 0x0A, SEEK_SET); offData = read_dword(fp);
    fseek(fp, 0x12, SEEK_SET);
    *width  = largeur = read_long(fp);
    *height = hauteur = read_long(fp);
    sbid        = read_word(fp); /* Planes */
    nbBitsParPixel = read_word(fp);
    lbid   = read_dword(fp);     /* Compress */
    uSizeImage = read_dword(fp);
    if (uSizeImage == 0) uSizeImage = (largeur*nbBitsParPixel + 7)/8*abs(hauteur);
    if ((bits = malloc(uSizeImage)) == NULL) {fclose(fp); return (NULL);}
    fseek(fp, offData, SEEK_SET);
    if (fread(bits, 1, uSizeImage, fp) < uSizeImage) {free(bits); fclose(fp); return (NULL);}

    /* Conversion BGR vers RGB */
    length = (largeur * 3 + 3) & ~3;
    for (y = 0; y < hauteur; y ++) for (ptr = bits + y * length, x = largeur; x > 0; x --, ptr += 3)
    { temp   = ptr[0]; ptr[0] = ptr[2]; ptr[2] = temp; }

    fclose(fp);
    return (bits);
}

/*
 * 'TextureLoad()' - Load a bitmap file and define it as a 1D or 2D texture.
 *
 * Returns > 0 on success or 0 on error.
 */

GLuint                           /* O - Texture object or 0 on error */
TextureLoad(char      *filename, /* I - Bitmap file to load */
            GLboolean alpha,     /* I - Generate alpha for bitmap */
            GLenum    minfilter, /* I - Minification filter */
            GLenum    magfilter, /* I - Magnification filter */
            GLenum    wrap)      /* I - Repeat or clamp */
    {
    GLubyte	*bits;           /* Bitmap RGB pixels */
    GLenum      type;            /* Texture type */
    GLuint      textobj = 0;         /* Texture object */
    long largeur, hauteur;

    /* Try loading the bitmap file... */
    bits = loadBmpPix(filename, &largeur, &hauteur);
    if (bits == NULL) return 0;

    if (hauteur == 1) type = GL_TEXTURE_1D;
    else type = GL_TEXTURE_2D;

    /* Create and bind a texture object */
    glGenTextures(1, &textobj);
    glBindTexture(type, textobj);
    /* Set texture parameters */
    glTexParameteri(type, GL_TEXTURE_MAG_FILTER, magfilter);
    glTexParameteri(type, GL_TEXTURE_MIN_FILTER, minfilter);
    glTexParameteri(type, GL_TEXTURE_WRAP_S, wrap);
    glTexParameteri(type, GL_TEXTURE_WRAP_T, wrap);
    glTexEnvi(type, GL_TEXTURE_ENV_MODE, GL_MODULATE);

    if (minfilter == GL_LINEAR || minfilter == GL_NEAREST)
        glTexImage2D(type, 0, 3, largeur, hauteur, 0, GL_RGB, GL_UNSIGNED_BYTE, bits);
    else if (type == GL_TEXTURE_1D)
        gluBuild1DMipmaps(type, 3, largeur, GL_RGB, GL_UNSIGNED_BYTE, bits);
    else
        gluBuild2DMipmaps(type, 3, largeur, hauteur, GL_RGB, GL_UNSIGNED_BYTE, bits);

    free(bits);
    return (textobj);
    }

