#include <sys/stat.h>
#include <GL/glu.h>
#include <math.h>
#include "mllglutil.h"
#include "visusim.h"

//-------------------------------------SimScene------------------------------------
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define DEG2RAD (M_PI/180.)

// static GLfloat eteinte[] = {0.0f, 0.0f, 0.0f, 1.0f};
static GLfloat faible[] = {0.2f, 0.2f, 0.2f, 0.0f};
static GLfloat puissante[] = {1.0f, 1.0f, 1.0f, 1.0f};

SimScene::SimScene()
{
    setWindowTitle(QObject::tr("VisuSim"));
    setFocusPolicy( Qt::StrongFocus);
    if (QGLFormat::hasOpenGL())
    {
        // Widget ordinaire mis en centralWidget (unique dans une QMainWindow)
        QWidget *centralWidget = new QWidget(this);
        setCentralWidget(centralWidget);
        // mais on le configure en QVBoxLayout
        QVBoxLayout *mainLayout = new QVBoxLayout();
        centralWidget->setLayout(mainLayout);

        // Timer pour fonction IDLE
        timer = new QTimer(this);
        connect(timer, SIGNAL(timeout()), this, SLOT(timeOutSlot()));
        timer->stop();
        dtime = 1;

        // LES CALLBACKS DE GESTION DES SOCKETS
        // * Reception acquitement demande de connexion
        connect(&tcpSocket, SIGNAL(connected()), this, SLOT(skSendRequest()));
        // * Reception fermeture connexion par le serveur
        connect(&tcpSocket, SIGNAL(disconnected()), this, SLOT(skConnectionClosedByServer()));
        // * Reception nouveau paquet de donnees
        connect(&tcpSocket, SIGNAL(readyRead()), this, SLOT(skReadNewData()));
        // * Reception erreur transmission
        connect(&tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(skError()));

        // LE MENU CONTEXTUEL
        // item "Help"
        QAction *actionHelp = new QAction(tr("&Help"), this);
        connect(actionHelp, SIGNAL(triggered()), this, SLOT(afficheHelpSlot()));
        addAction(actionHelp);
        // item "Sauve Image"
        QAction *actionGrab = new QAction(tr("&Sauve Image"), this);
        connect(actionGrab, SIGNAL(triggered()), this, SLOT(grabImageSlot()));
        addAction(actionGrab);
        // item "Lecture Fichier"
        actionLectFich = new QAction(tr("&Lecture Fichier"), this);
        connect(actionLectFich, SIGNAL(triggered()), this, SLOT(initLectFichSlot()));
        addAction(actionLectFich);
        // item "Lecture Reseau"
        actionLectNet = new QAction(tr("&Lecture Reseau"), this);
        connect(actionLectNet, SIGNAL(triggered()), this, SLOT(initLectNetSlot()));
        addAction(actionLectNet);

        // Activation Menu contextuel
        setContextMenuPolicy(Qt::ActionsContextMenu);

        //Les boutons et sliders
        label = new QLabel(tr("000000/000000"));
        label->setMinimumSize(label->sizeHint());
        slider = new QSlider(Qt::Horizontal);
        play_pauseButton = new QPushButton("PLAY");
        initButton = new QPushButton("INIT");

        play_pauseButton->setDisabled(true);
        initButton->setDisabled(true);
        slider->setDisabled(true);

        connect(play_pauseButton,SIGNAL(clicked()),this,SLOT(playpauseSlot()));
        connect(initButton,SIGNAL(clicked()),this,SLOT(initButtonSlot()));

        statusLabel = new QLabel(tr("Mode local : Touches X,Y,Z... : Menu contextuel actif"));

        fdata = NULL;

        // Le QGLWidget SimWidget, ses boutons et son slider
        wGL = new SimWidget(this, this);

        // est positionne au centre et ses controles en ligne en dessous
        mainLayout->addWidget(wGL);
        QHBoxLayout *basLayout = new QHBoxLayout();
        mainLayout->addLayout(basLayout);
        basLayout->addWidget(play_pauseButton);
        basLayout->addWidget(initButton);
        basLayout->addWidget(label);
        basLayout->addWidget(slider);
        statusBar()->addWidget(statusLabel);

        // Exemple de labelHelp (à surcharger éventuellement)
        labelHelp = new QLabel(
                tr(
                        "<h5>ACTION DES TOUCHES DU CLAVIER :</h5>"
                        "<ul><li>p(minus) pour projection orthogonale</li>"
                        "<li>P(majus) pour perspective</li></ul>"
                        "<h5>Mouvements observateur :</h5>"
                        "<ul>"
                        "<li>Touches e-E     : +/- éloignement</li>"
                        "<li>Touches Deb-Fin : Droite - Gauche</li>"
                        "<li>Page Haut-Bas   : Haut - Bas</li>"
                        "<li>Fleches &gt; - &lt; : rotation azimut</li>"
                        "<li>Fleches ^ - V   : rotation site</li>"
                        "<li>ESC             : point de vue initial</li></ul>"
                        ), 0);
        labelHelp->setWindowTitle("Aide visu basique");
    }
    else
    {
        QLabel *label = new QLabel(tr(" This system has no OpenGL support !! "));
        setCentralWidget(label);
    }

    // On démarre en mode LOCAL + PAUSE
    devInput = LOCAL;
    modePlay = PAUSE;
}

// Gestion ticks horloge
void SimScene::timeOutSlot()
{
    if (devInput != VISUFICH) return;

    if (nech == nechtot) // Si arrive a la fin
    {
        // On passe en pause et on remet le compteur d'echantillon a zero.
        playpauseSlot(); nech = 0;
        return;
    }

    // Sinon on affiche l'echantillon et on incremente le compteur
    if (nbpars == lecSampleSlot(nech))
    {
        // buffer de donnees Ok
        slider->setValue(nech);
        nech++;
    }
    else // Echec lecture echantillon
    {
        if(nech == 0)
        { // Le fichier est vide
            devInput = LOCAL;
            return;
        }
        playpauseSlot(); nech = 0;
    }
    return;
}

// Envoie de la requete "codeConnexion" au serveur
void SimScene::skSendRequest()
{
    nechtot = 0;
    nextBlockSize = 0;
    if (fdata) fclose(fdata);
    fdata = fopen("SimScene.txt","wb");

    QByteArray block; // Le block qui sera envoyé
    QDataStream out(&block, QIODevice::WriteOnly);
    out << quint16(0) << QObject::tr("VisuSim") ;
    out.device()->seek(0);
    out << quint16(block.size() - sizeof(quint16));
    tcpSocket.write(block);
}

// Traitement signal disconnected()
void SimScene::skConnectionClosedByServer()
{
    if (nextBlockSize != 0xFFFF)
    {
        skFermeture(tr("Erreur: Connexion fermee par le serveur"));
    }
    else
    {
        skFermeture(QString(tr("Fin de connexion")));
    }
}

// Traitement signal error()
void SimScene::skError()
{
    skFermeture(tcpSocket.errorString());
}


// Boucle de lecture des paquets de donnees sur socket
void SimScene::skReadNewData()
{
    statusLabel->setText(tr("Lecture trajectoire par socket"));
    QDataStream in(&tcpSocket);
    forever
    {
        long val;
        char ligne[24];
        double v, data[nbpars];

        if (nextBlockSize == 0)
        {
            if (tcpSocket.bytesAvailable() < sizeof(quint16)) break;
            in >> nextBlockSize;
        }

        if (nextBlockSize == 0xFFFF)
        {
            skFermeture(tr("Trouve %1 ligne(s)").arg(nechtot));
            break;
        }

        // sortie pour attente d'un block complet
        if (tcpSocket.bytesAvailable() < nextBlockSize)  break;

        // Preparation enregistrement sur fichier
        val = ftell(fdata);
        if (nechtot < NECHMAX) tabIndex[nechtot++] = val;

        // Lecture socket et écriture sur fichier
        in >> v; fprintf(fdata, " %lf", v);
        for (int i = 0; i < nbpars; i++) {in >> v; data[i] = v; fprintf(fdata, " %lf", v);}
        fprintf(fdata,"\n");

        // Affichage visu en meme temps
        setConfig(data);
        sprintf(ligne,"%06d/%06d", nechtot-1, nechtot-1);
        label->setText(ligne);

        nextBlockSize = 0;
        // Pour permettre la lecture du bouton d'interruption de connexion
        // sans pour autant laisser passer un nouveau dataReady qui produirait
        // une re-entree dans la routine alors qu'elle est en cours.
        //qApp->processEvents(QEventLoop::ExcludeSocketNotifiers);
    }
}

void SimScene::skFermeture(QString mes)
{
    statusLabel->setText(mes);
    if (fdata) fclose(fdata);
    tcpSocket.close();
    if (nechtot > 1)
    {
        fdata = fopen("SimScene.txt","rb");
        initReplay(VISUFICH, PAUSE);
    }
    else initReplay(LOCAL);

}

// Initialisation avant mode replay du fichier fdata
void SimScene::initReplay(ModeInput mi, ModePlay mp)
{
    // On remet le menu pour permettre la lecture d'un nouveau fichier
    actionLectFich->setText(tr("&Lecture Fichier"));
    actionLectFich->setEnabled(true);
    actionLectNet->setText(tr("&Lecture Reseau"));
    actionLectNet->setEnabled(true);
    devInput = mi;
    if (devInput == VISUFICH)
    { //On passe en mode PLAY
        char ligne[24];
        fseek(fdata,0,0);
        nech = 0;
        slider->setMaximum(nechtot-1);
        slider->setValue(nech);
        sprintf(ligne,"%06d/%06d", nech, nechtot-1);
        label->setText(ligne);
        play_pauseButton->setDisabled(false);
        modePlay = (mp == PLAY ? PAUSE : PLAY) ;
        playpauseSlot(); // va commuter dans l'autre mode
    }
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement touches et fleches pour actions diverses                               */
/*----------------------------------------------------------------------------------*/
void SimScene::keyPressEvent(QKeyEvent *event)
{
    int key = event->key();
    bool majuscule = (Qt::SHIFT == event->modifiers());

    // Fleches modification point de vue toujours actives
    if (wGL->keyObserver(key, majuscule)) return;

    // Autres touches actives qu'en local ou en pause, sinon on passe
    if (devInput == LOCAL || modePlay == PAUSE) keyConfig(key, majuscule) ;
    else  QMainWindow::keyPressEvent(event);
}

// Gestion bouton Play/Pause
void SimScene::playpauseSlot()
{
    if (modePlay == PLAY) // On est en play et on a appuye sur pause
    {
        // On passe en pause
        timer->stop();
        modePlay = PAUSE;
        play_pauseButton->setText("PLAY");// On met le texte pour pouvoir repasser en Play
        slider->setDisabled(false);       // Le slider est receptif
        initButton->setDisabled(false);   // Le bouton Init est receptif
        // Le slider controle le mouvement
        connect(slider,SIGNAL(valueChanged(int)),this,SLOT(lecSampleSlot(int)));
        setFocusPolicy( Qt::StrongFocus); // On prend en compte les entrees clavier
        statusLabel->setText(tr("Mode Pause : Menu contextuel actif"));
    }
    else if (modePlay == PAUSE)// On est en pause et on a appuye sur play
    {
        // On passe en play
        // Le slider ne controle plus le mouvement
        disconnect(slider,SIGNAL(valueChanged(int)),this,SLOT(lecSampleSlot(int)));
        slider->setDisabled(true);         // Le slider n'est plus receptif
        modePlay = PLAY;
        play_pauseButton->setText("PAUSE");// On met le texte pour pouvoir repasser en Pause
        setFocusPolicy( Qt::NoFocus);      // On ne prend pas en compte les entrees clavier
        initButton->setDisabled(true);     // le bouton Init n'est pas receptif
        timer->start(dtime);
        statusLabel->setText(tr("Mode Play : Menu contextuel inactif"));
    }
}

// Gestion bouton INIT (a cote du PLAY/PAUSE)
void SimScene::initButtonSlot()
{
    slider->setValue(0);
    // lecSampleSlot(0); // inutile car en pause le slider est connecte a lecSampleSlot
    initButton->setDisabled(true);
}

// Lecture echantillon numero m dans le fichier
int SimScene::lecSampleSlot(int m)
{
    char ligne[24];
    double v, data[nbpars];

    int n = -1;
    /* Positionnement au debut de l'echantillon m */
    if (m < 0 || m > nechtot)
        return n;
    fseek(fdata,tabIndex[m],0);
    if (NULL != fgets(bufdata, CHARMAX, fdata)) n = strlen(bufdata);

    QByteArray ba(bufdata, n);
    QTextStream in(ba, QIODevice::ReadOnly);
    in >> v; // Lecture du temps inutilisé pour le moment
    for (n = 0; n < nbpars;  )
    {
        in >> data[n++];
        if(in.atEnd())
            break;
    }

    if (n == nbpars)
    {
        nech = m; // pour l'affichage de nech/nechtot
        setConfig(data);
        sprintf(ligne,"%06d/%06d", nech, nechtot-1);
        label->setText(ligne);
    }
    else
    {
        nechtot = m; // Reduction du nombre d'echantillons
        slider->setMaximum(nechtot-1);
    }
    return n;
}


// Affichage Infos Aide
void SimScene::afficheHelpSlot()
{
    labelHelp->show();
}

// Enregistrement d'image-ecran sous format jpg
void SimScene::grabImageSlot()
{
    QImage image = wGL->grabFrameBuffer();
    QString filename = QFileDialog::getSaveFileName(this,tr("Sauve Image"),".",tr("Fichier image (*.jpg)"));
    if (!filename.isEmpty()) image.save(filename);
}

// Traitement appui sur Lecture_Fichier/Arret_Lecture
void SimScene::initLectFichSlot()
{
    initLecture(FICHIER);
}

// Traitement appui sur Lecture_Net/Arret_Lecture
void SimScene::initLectNetSlot()
{
    initLecture(RESEAU);
}

// Traitement fenetre iconifiee
void SimScene::hideEvent(QHideEvent * /* event */)
{
    if (devInput == VISUFICH && modePlay == PLAY) timer->stop();
}
// Traitement fenetre affichee
void SimScene::showEvent(QShowEvent * /* event */)
{
    if (devInput == VISUFICH && modePlay == PLAY) timer->start(dtime);
}

// Traitement appui sur Lecture Fichier ou Net/Arret...
void SimScene::initLecture(OriInput origin)
{
    switch(devInput)
    {
    case LECFICH : // On lisait le fichier sur disque : on arrete cette lecture
        devInput = VISUFICH; // On se met en mode visaulisation de ce qui a été lu
        // On remet le menu pour permettre la lecture d'un nouveau fichier
        actionLectFich->setText(tr("&Lecture Fichier"));
        // On autorise egalement la lecture reseau
        actionLectNet->setEnabled(true);
        break;

    case LECNET : // On était en lecture reseau : on arrete cette lecture
        skFermeture(tr("Demande fin lecture reseau"));
        break;
    case VISUFICH : // On visualisait, on passe à une autre lecture
    case LOCAL : // On etait en local, on passe en lecture fichier
        // On désactive le slider
        disconnect(slider,SIGNAL(valueChanged(int)),this,SLOT(lecSampleSlot(int)));
        slider->setDisabled(true);         // Le slider n'est plus receptif
        switch(origin)
        {
        case RESEAU : // Demande Lecture reseau
            // On interdit la demande de lecture d'un nouveau fichier
            actionLectFich->setDisabled(true);
            //On se positionne en mode Lecture reseau
            devInput = LECNET;
            statusLabel->setText(tr("Connexion au serveur..."));
            actionLectNet->setText(tr("&Arreter Reseau"));
            skConnectToHost(QHostAddress::LocalHost, 6178);
            break;
        case FICHIER : // Demande Lecture fichier
            // On interdit la demande de lecture reseau
            actionLectNet->setDisabled(true);
            QString NomFich = QFileDialog::getOpenFileName (NULL,
                                         QObject::tr("Fichier data visu"),".");
            if(!NomFich.isEmpty())
            {
                if (fdata) fclose(fdata); fdata = NULL;
                fdata = fopen(NomFich.toLocal8Bit().data(),"rb");
                if (fdata)
                {
                    long val;
                    struct stat statbuf;

                    statusLabel->setText(tr("Lecture fichier..."));
                    fstat(fileno(fdata), &statbuf);

                    //On se positionne en mode Lecture ficher
                    devInput = LECFICH;
                    //On offre le menu arret lecture
                    actionLectFich->setText(tr("&Arreter Lecture"));
                    //Constitution table d'index des echantillons du fichier
                    nechtot = 0;
                    tabIndex[nechtot++] = 0;

                    while (devInput == LECFICH && nechtot < NECHMAX && NULL != fgets(bufdata, CHARMAX, fdata))
                    {
                        if (bufdata[0]=='#') continue;
                        val = ftell(fdata);
                        tabIndex[nechtot++] = val;
//                        if (nechtot%1000 == 0)
//                        {
//                            char ligne[24];
//                            sprintf(ligne,"%06ld/%06ld", val/1024, statbuf.st_size/1024);
//                            label->setText(ligne);
//                            // Pour interrompre éventuellement la lecture
//                            qApp->processEvents();
//                        }
                    }
                    initReplay(VISUFICH, PAUSE);
                }
            }
            // On rétablit la possibilite de lecture reseau
            actionLectNet->setEnabled(true);
            break;
        }
        break;
    }
}

//-------------------------------------SimWidget------------------------------------
// LE CONSTRUCTEUR DU WIDGET CENTRAL
SimWidget::SimWidget(SimScene *s, QWidget *parent) : QGLWidget(parent)
{
    scene = s;
    // Format de la fenêtre
    setFormat(QGLFormat(QGL::DoubleBuffer | QGL::DepthBuffer | QGL::Rgba));
    // Initialisation point de vue de l'observateur (à surcharger éventuellement)
#define ELOING 10.
#define SITE 20.0
#define AZIMUT -30.0
    Perspective = 1;    /* Flag utilisation perspectives  */
    AngleVueVertical = 30. ; /* en degrés */
    DistCam = DistCamIni = ELOING; /* de l'observateur */
    AziCam = AziCamIni = AZIMUT;
    SitCam = SitCamIni = SITE; /* Direction de l'observateur */
    Wdroit = 0.; Whaut = 0.; /* Déplacement écran */
    flagZvertical = true;
}


// Dimension par defaut de la fenetre openGL 720x576 (pour standard DVD)
QSize SimWidget::sizeHint() const
{
    //On crée un objet QSize ayant la valeur par défaut retournée par sizeHint()
    QSize size = QGLWidget::sizeHint();
    size.rheight() = 576;
    size.rwidth() = 720;
    return size;
}

void SimWidget::resizeGL(int w, int h)
{
    width1 = w;  height1 = h;
}

void SimWidget::paintGL()
{
    double Dx,Dy, Infini = 1000.*DistCam;
    if (height1 == 0) return ;

    /****************************/
    /* Initialisation generale  */
    /****************************/
    glViewport(0,0,width1,height1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

    /*********************************************/
    /*  Definition fenetre et mode de projection */
    /*********************************************/
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity(); /* la matrice identite est chargee comme matrice de projection */
    if(Perspective)
        gluPerspective(AngleVueVertical, 1.*width1/height1, 0.01, Infini);
    else
    {
        Dy = DistCam*tan(0.5*AngleVueVertical*M_PI/180.);
        Dx = Dy*width1/height1;
        glOrtho(-Dx, Dx, -Dy, Dy, 0.01, Infini);
    }

    /*******************************/
    /* Mise en place repere scene  */
    /*******************************/
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    /* Positionnement inverse de l'observateur */
    glTranslated(Wdroit,Whaut, -DistCam);

    glRotatef(SitCam, 1.0, 0.0, 0.0);
    glRotatef(AziCam, 0.0, 1.0, 0.0);
    if(flagZvertical) glRotatef(-90., 1.0, 0.0, 0.0); /* Repère avec Z vertical */

    scene->redraw();
}

// Initialisation OpenGL
void SimWidget::initializeGL()
{
    /* Préparation des dessins */
    scene->create();
}

/*----------------------------------------------------------------------------------*/
/* Callback traitement fleches pour modifier le point de vue                        */
/* ^ - V               : +/- angle Site                                             */
/* > - <               : +/- angle Azimut                                           */
/* Home - End          : fenetre vers la gauche - droite                            */
/* Page-Up - Page-Down : fenetre vers le haut, bas                                  */
/* e - E               : eloigne ou approche l'observateur                          */
/* Esc                 : point de vue initial                                       */
/* p, P                : Vue avec ou sans perspective                               */
/*----------------------------------------------------------------------------------*/
bool SimWidget::keyObserver(int key, bool majuscule)
{
    bool ok = false;
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj;
    double DeltaA = 2.5, DeltaL = DistCamIni/100.;

    switch (key)
    {
    case Qt::Key_Up :
        SitCam += DeltaA ;
        ok = TRUE;
        break ;
    case Qt::Key_Down :
        SitCam -= DeltaA ;
        ok = TRUE;
        break ;
    case Qt::Key_Left :
        AziCam += DeltaA ;
        if (AziCam > 180.) AziCam -= 360. ;
        ok = TRUE;
        break ;
    case Qt::Key_Right :
        AziCam -= DeltaA ;
        if (AziCam < -180.) AziCam += 360. ;
        ok = TRUE;
        break ;
    case Qt::Key_Home : /* fenetre a droite */
        Wdroit -= DeltaL ;
        ok = TRUE;
        break ;
    case Qt::Key_End : /* fenetre a gauche*/
        Wdroit += DeltaL ;
        ok = TRUE;
        break ;
    case Qt::Key_PageUp : /* fenetre en haut */
        Whaut += DeltaL ;
        ok = TRUE;
        break ;
    case Qt::Key_PageDown : /* fenetre en bas */
        Whaut -= DeltaL ;
        ok = TRUE;
        break ;
    case Qt::Key_Escape : /* Point de vue initial */
        Wdroit = 0. ;
        Whaut = 0.;
        DistCam = DistCamIni;
        AziCam = AziCamIni; SitCam = SitCamIni;
        // ok = TRUE;
        updateGL();
        break ;
    case Qt::Key_E : /* DistCam */
        DistCam -= DeltaL*smaj ;
        ok = TRUE;
        break ;
    case Qt::Key_P : /* Perspective */
        Perspective = maj ;
        ok = TRUE;
        break;
    }
    if (ok) updateGL();
    return ok;
}
