#ifndef VISUSIM_H
#define VISUSIM_H
#include <QtWidgets>
#include <QGLWidget>
#include <QtNetwork>
#include <QTcpSocket>


class SimWidget;

#define NECHMAX 20000 // Nombre max d'echantillons sauvegardes dans les fichiers
#define CHARMAX 256 // Nombre max de caracteres par ligne texte echantillon
typedef enum {LOCAL, LECFICH, LECNET, VISUFICH} ModeInput;
typedef enum {FICHIER, RESEAU} OriInput;
typedef enum {PLAY, PAUSE} ModePlay;
//-------------------------------------SimScene----------------------------------
class SimScene : public QMainWindow
{
    Q_OBJECT

public:
    SimScene();
    virtual void redraw(void) = 0;
    virtual void create() = 0;
    virtual void setConfig(double *data) = 0;
    virtual bool keyConfig(int, bool) = 0;
    QLabel *labelHelp;

    SimWidget *wGL;

private :
    int dtime;
    QTimer *timer;
    QSlider *slider;
    QLabel *label;
    QPushButton *play_pauseButton;
    QPushButton *initButton;
    QLabel *statusLabel;
    QAction *actionLectFich;
    QAction *actionLectNet;
    OriInput oriInput;
    ModePlay modePlay;
    int nech, nechtot;
    FILE *fdata;
    long tabIndex[NECHMAX];
    char bufdata[CHARMAX+2];
    void initLecture(OriInput);
    void initReplay(ModeInput mi, ModePlay mp = PAUSE);
// Pour les sockets
    QTcpSocket tcpSocket;
    quint16 nextBlockSize;
    void skConnectToHost(const QHostAddress host, quint16 port)
                        {tcpSocket.connectToHost(host, port);}
    void skFermeture(QString);




private slots:
    void timeOutSlot();
    void afficheHelpSlot();
    void grabImageSlot();
    void initLectFichSlot();
    void initLectNetSlot();
    void playpauseSlot();
    void initButtonSlot();
    int lecSampleSlot(int m);
    void skSendRequest();
    void skReadNewData();
    void skConnectionClosedByServer();
    void skError();

protected:
    void showEvent(QShowEvent *event);
    void hideEvent(QHideEvent *event);
    void keyPressEvent(QKeyEvent *event);
    ModeInput devInput;
    int nbpars;
};

//-----------------------------------SimWidget-------------------------------------
class SimWidget : public QGLWidget
{

public:
    SimWidget(SimScene *scene, QWidget *parent);
    QSize sizeHint () const ;
    // suivants publics pour acces depuis SimScene
    bool keyObserver(int, bool);
    double DistCam, AziCam, SitCam;
    double DistCamIni, AziCamIni, SitCamIni;
    int width1, height1;
    bool flagZvertical;

private :
    SimScene *scene;
    int	Perspective;    /* Flag utilisation perspectives  */
    double AngleVueVertical; /* en degrés */
    double Wdroit, Whaut; /* Déplacement écran */

protected:
    void initializeGL();
    void resizeGL(int width, int height);
    void paintGL();
};
#endif
